from .AbaqusVariable import AbaqusVariable


class AbaqusMaterialElastic(AbaqusVariable):
    modulus: float = None
    poisson_ratio: float = None

    def __init__(self, modulus: float = None, poisson_ratio: float = None):
        self.setParameters(modulus, poisson_ratio)

    def setModulus(self, modulus: float):
        self.modulus = modulus

    def setPoissonRatio(self, poisson_ratio: float):
        self.poisson_ratio = poisson_ratio

    def setParameters(self, modulus: float, poisson_ratio: float):
        self.modulus, self.poisson_ratio = modulus, poisson_ratio


class AbaqusMaterial(AbaqusVariable):
    density: float = None
    elastic: AbaqusMaterialElastic

    def __init__(self, modulus: float = None, poisson_ratio: float = None, density: float = None):
        self.elastic = AbaqusMaterialElastic()
        self.setParameters(modulus, poisson_ratio, density)

    def setParameters(self, modulus: float, poisson_ratio: float, density: float):
        self.setDensityParameters(density)
        self.setElasticParameters(modulus, poisson_ratio)

    def setDensityParameters(self, density: float):
        self.density = density

    def setElasticParameters(self, modulus: float, poisson_ratio: float):
        self.elastic.setParameters(modulus, poisson_ratio)


import typing

from .AbaqusVariable import AbaqusVariable


class AbaqusGeometry(AbaqusVariable):
    values: tuple[typing.Any] = None

    def __init__(self, *values: typing.Any):
        self.setParameters(*values)

    def setParameters(self, *values: typing.Any):
        self.values = values



from .AbaqusVariable import AbaqusVariable


class AbaqusOutput(AbaqusVariable):
    variables: list[str] = []

    def __init__(self, variables: list[str]):
        self.setParameters(variables)

    def setParameters(self, variables: list[str]):
        self.variables = variables

    def addOutputVariable(self, variable: str):
        self.variables.append(variable)

    def addOutputVariables(self, variables: list[str]):
        self.variables.extend(variables)


class AbaqusFieldOutput(AbaqusOutput):
    frequency: int = 1

    def __init__(self, variables: list[str], frequency: int = 1):
        super(AbaqusFieldOutput, self).__init__(variables)
        self.frequency = frequency

    def setParameters(self, variables: list[str], frequency: int = 1):
        super(AbaqusFieldOutput, self).setParameters(variables)
        self.frequency = frequency


class AbaqusHistoryOutput(AbaqusOutput):
    num_intervals: int = 20

    def __init__(self, variables: list[str], num_intervals: int = 1):
        super(AbaqusHistoryOutput, self).__init__(variables)
        self.num_intervals = num_intervals

    def setParameters(self, variables: list[str], num_intervals: int = 1):
        super(AbaqusHistoryOutput, self).setParameters(variables)
        self.num_intervals = num_intervals



import os
import typing

from .AbaqusGeometry import AbaqusGeometry
from .AbaqusLoad import AbaqusPressure, AbaqusPredefinedStress, AbaqusDisplacement, AbaqusVelocity
from .AbaqusMaterial import AbaqusMaterial
from .AbaqusMesh import AbaqusMesh, AbaqusMeshMethod
from .AbaqusStep import (AbaqusBaseStepIncrementationType, AbaqusGeneralStaticStep, AbaqusExplicitDynamicStep,
                         AbaqusCalculationMethod)
from .AbaqusSubroutine import AbaqusSubroutine, AbaqusSubroutineType
from.AbaqusOutput import AbaqusFieldOutput, AbaqusHistoryOutput


class AbaqusModelBase:
    calculation_method = AbaqusCalculationMethod.Standard
    geometries: dict[str, AbaqusGeometry] = {}
    materials: dict[str, AbaqusMaterial] = {}
    steps: dict[str, typing.Union[AbaqusGeneralStaticStep, AbaqusExplicitDynamicStep]] = {}
    loads: dict[str, typing.Union[AbaqusPressure, AbaqusPredefinedStress, AbaqusDisplacement, AbaqusVelocity]] = {}
    subroutines: dict[str, AbaqusSubroutine] = {}
    meshes: dict[str, AbaqusMesh] = {}
    outputs: dict[str, typing.Union[AbaqusFieldOutput, AbaqusHistoryOutput]] = {}

    old_work_directory = os.getcwd()
    work_directory: str = os.getcwd()
    model_name = 'dst'
    output_name = 'output'

    def __init__(self):
        pass

    def WorkDirectory(self, work_directory: str):
        if work_directory == '':
            return

        if not os.path.exists(work_directory):
            os.mkdir(work_directory)
        os.chdir(work_directory)
        self.work_directory = os.getcwd()

    def ModelName(self, model_name: str):
        self.model_name = model_name

    def OutputName(self, output_name: str):
        self.output_name = output_name

    def setCalculationMethod(self, calculation_method=AbaqusCalculationMethod.Standard):
        self.calculation_method = calculation_method

    def calculationMethodString(self):
        return 'STANDARD' if self.calculation_method == AbaqusCalculationMethod.Standard else 'EXPLICIT'

    def Geometry(self, name: str, *values: typing.Any):
        if name in self.geometries.keys:
            self.geometries[name].setParameters(*values)
        else:
            self.geometries[name] = AbaqusGeometry(values)

    def Material(self, name: str, modulus: float = None, poisson_ratio: float = None, density: float = None):
        if name in self.materials.keys():
            self.materials[name].setParameters(modulus, poisson_ratio, density)
        else:
            self.materials[name] = AbaqusMaterial(modulus, poisson_ratio, density)

    def Pressure(self, name: str, amplitude: typing.Union[float, typing.Iterable]):
        if name in self.loads.keys() and type(self.loads[name]) == AbaqusPressure:
            self.loads[name].setParameters(amplitude)
        else:
            self.loads[name] = AbaqusPressure(amplitude)

    @typing.overload
    def PredefinedStress(self, name: str, predefined_stress: typing.Iterable):
        pass

    @typing.overload
    def PredefinedStress(self, name: str, sig11: float, sig22: float, sig33: float, sig12: float, sig13: float,
                         sig23: float):
        pass

    def PredefinedStress(self, name: str, *args):
        if name in self.loads.keys() and type(self.loads[name]) == AbaqusPredefinedStress:
            self.loads[name].setParameters(*args)
        else:
            self.loads[name] = AbaqusPredefinedStress(*args)

    @typing.overload
    def Displacement(self, name: str, displacement: typing.Iterable = None):
        pass

    @typing.overload
    def Displacement(self, name: str, u1: float, u2: float, u3: float, ur1: float, ur2: float, ur3: float):
        pass

    def Displacement(self, name: str, *args):
        if name in self.loads.keys() and type(self.loads[name]) == AbaqusDisplacement:
            self.loads[name].setParameters(*args)
        else:
            self.loads[name] = AbaqusDisplacement(*args)

    @typing.overload
    def Velocity(self, name: str, displacement: typing.Iterable):
        pass

    @typing.overload
    def Velocity(self, name: str, v1: float, v2: float, v3: float, vr1: float, vr2: float, vr3: float):
        pass

    def Velocity(self, name: str, *args):
        if name in self.loads.keys() and type(self.loads[name]) == AbaqusVelocity:
            self.loads[name].setParameters(*args)
        else:
            self.loads[name] = AbaqusVelocity(*args)

    @typing.overload
    def Mesh(self, name: str, method: AbaqusMeshMethod.ByNumber, seeds_number: int):
        pass

    @typing.overload
    def Mesh(self, name: str, method: AbaqusMeshMethod.BySize, seeds_size: float):
        pass

    def Mesh(self, name: str, method: AbaqusMeshMethod, seeds: typing.Union[int, float]):
        if name in self.meshes.keys():
            self.meshes[name].setParameters(method, seeds)
        else:
            self.meshes[name] = AbaqusMesh(method, seeds)

    @typing.overload
    def GeneralStaticStep(self, name: str, time_period: float = 1.0, description: str = '',
                          incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                          maximal_increments: int = 100, initial_increment_size: float = 1.0,
                          minimal_increment_size: float = 1e-5, maximal_increment_size: float = 1.0):
        pass

    @typing.overload
    def GeneralStaticStep(self, name: str, time_period: float = 1.0, description: str = '',
                          incrementation_type=AbaqusBaseStepIncrementationType.Fixed,
                          maximal_increments: int = 100, fixed_increment_size: float = 0.1):
        pass

    def GeneralStaticStep(self, name: str, time_period: float = 1.0, description: str = '',
                          incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                          *args, **kwargs):
        if name in self.steps.keys() and type(self.steps[name]) == AbaqusGeneralStaticStep:
            self.steps[name].setParameters(time_period, description, incrementation_type, *args, **kwargs)
        else:
            self.steps[name] = AbaqusGeneralStaticStep(time_period, description, incrementation_type, *args, **kwargs)

    @typing.overload
    def ExplicitDynamicStep(self, name: str, time_period: float = 1.0, description: str = '',
                            linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2,
                            scaling_factor: float = 1.0, incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                            maximal_time_increment: typing.Union[float, str] = 'UNLIMITED'):
        pass

    @typing.overload
    def ExplicitDynamicStep(self, name: str, time_period: float = 1.0, description: str = '',
                            linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2,
                            scaling_factor: float = 1.0, incrementation_type=AbaqusBaseStepIncrementationType.Fixed,
                            increment_size: typing.Union[float, str] = 'ELEMENT_BY_ELEMENT_INCREMENTATION'):
        pass

    def ExplicitDynamicStep(self, name: str, time_period: float = 1.0, description: str = '',
                            linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2,
                            scaling_factor: float = 1.0, incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                            *args, **kwargs):
        if name in self.steps.keys() and type(self.steps[name]) == AbaqusExplicitDynamicStep:
            self.steps[name].setParameters(time_period, description, linear_bulk_viscosity, quad_bulk_viscosity,
                                           scaling_factor, incrementation_type, *args, **kwargs)
        else:
            self.steps[name] = AbaqusExplicitDynamicStep(time_period, description, linear_bulk_viscosity,
                                                         quad_bulk_viscosity, scaling_factor, incrementation_type,
                                                         *args, **kwargs)

    def Subroutine(self, name: str, subroutine_type=AbaqusSubroutineType.FRIC, subroutine_filepath: str = '',
                   n_state_dependent_variables: int = 0, properties: typing.Iterable = None):
        if name in self.subroutines.keys():
            self.subroutines[name].setParameters(subroutine_type, subroutine_filepath, n_state_dependent_variables,
                                                 properties)
        else:
            self.subroutines[name] = AbaqusSubroutine(subroutine_type, subroutine_filepath, n_state_dependent_variables,
                                                      properties)

    def FieldOutput(self, name: str, variables: list[str], frequency: int = 1):
        if name in self.outputs.keys() and type(self.outputs[name]) == AbaqusFieldOutput:
            self.outputs[name].setParameters(variables, frequency)
        else:
            self.outputs[name] = AbaqusFieldOutput(variables, frequency)

    def DefaultFieldOutput(self, variables: list[str], frequency: int = 1):
        self.FieldOutput('F-Output-1', variables, frequency)

    def HistoryOutput(self, name: str, variables: list[str], num_intervals: int = 10):
        if name in self.outputs.keys() and type(self.outputs[name]) == AbaqusHistoryOutput:
            self.outputs[name].setParameters(variables, num_intervals)
        else:
            self.outputs[name] = AbaqusHistoryOutput(variables, num_intervals)

    def DefaultHistoryOutput(self, variables: list[str], num_intervals: int = 10):
        self.HistoryOutput('H-Output-1', variables, num_intervals)

    def generateAbaqusPythonScript(self):
        pass

    def submit(self):
        pass
    
    def resetWorkDirectory(self):
        """Reset the work wirectory to your own outer directory, since the work directory has been set to 
        AbaqusModelBase.work_directory or AbaqusDirectShearTest.work_directory that is set by the method 
        WorkDirectory or specified by the work_driectory parameter.
        """
        os.chdir(self.old_work_directory)



import typing
from enum import IntEnum

from .AbaqusVariable import AbaqusVariable


class AbaqusMeshMethod(IntEnum):
    ByNumber = 0
    BySize = 1


class AbaqusMesh(AbaqusVariable):
    method = AbaqusMeshMethod.ByNumber
    seeds_number: int = 1
    seeds_size: float = 0.01

    @typing.overload
    def __init__(self, method=AbaqusMeshMethod.ByNumber, seeds_number: int = 1):
        pass

    @typing.overload
    def __init__(self, method=AbaqusMeshMethod.BySize, seeds_size: float = 0.01):
        pass

    def __init__(self, method=AbaqusMeshMethod.ByNumber, seeds: typing.Union[int, float] = 1):
        self.method = method
        if method == AbaqusMeshMethod.BySize:
            self.setParameters(method, seeds)
        else:
            self.setParameters(method, seeds)

    def methodString(self):
        if self.method == AbaqusMeshMethod.ByNumber:
            return 'NUMBER'
        else:
            return 'SIZE'

    @typing.overload
    def setParameters(self, method=AbaqusMeshMethod.ByNumber, seeds_number: int = 1):
        pass

    @typing.overload
    def setParameters(self, method=AbaqusMeshMethod.BySize, seeds_size: float = 0.01):
        pass

    def setParameters(self, method=AbaqusMeshMethod.ByNumber, seeds: typing.Union[int, float] = 1):
        self.method = method
        if method == AbaqusMeshMethod.BySize:
            self.seeds_size = seeds
        else:
            self.seeds_number = seeds



import os
import re
import shutil
import typing

import numpy as np
from matplotlib import pyplot as plt

from ..ababase.AbaqusGeometry import AbaqusGeometry
from ..ababase.AbaqusMesh import AbaqusMeshMethod
from ..ababase.AbaqusModelBase import AbaqusModelBase
from ..ababase.AbaqusStep import AbaqusCalculationMethod
from ..ababase.AbaqusSubroutine import AbaqusSubroutineType


class AbaqusDirectShearTestGeometry(AbaqusGeometry):
    length: float = None
    width: float = None
    height: float = None

    def __init__(self, length: float = None, width: float = None, height: float = None):
        super().__init__(length, width, height)
        self.setParameters(length, width, height)

    def setParameters(self, length: float, width: float, height: float):
        super().setParameters(length, width, height)
        self.length, self.width, self.height = length, width, height


class AbaqusDirectShearTest(AbaqusModelBase):
    """
    This is a python class to manage the information of the direct shear test, and to generate python script to build
    the abaqus model. You can use this class to

    - generate python script to build abaqus direct shear test model
    - submit the abaqus job using the abaqus command
    - generate python script to obtain the output data of the direct shear test model
    - plot some figures using the output data
    """
    geometries: dict[str, AbaqusDirectShearTestGeometry] = {}

    def __init__(self, calculation_method=AbaqusCalculationMethod.Standard,
                 soil_geometry: tuple[float, float, float] = (0.025, 0.025, 0.025),
                 solid_geometry: tuple[float, float, float] = (0.05, 0.05, 0.05),
                 soil_material: tuple[float, float, typing.Optional[float]] = (4e4, 0.0, None),
                 solid_material: tuple[float, float, typing.Optional[float]] = (2e8, 0.0, None),
                 soil_mesh_method=AbaqusMeshMethod.ByNumber, soil_seeds: typing.Union[int, float] = 1,
                 solid_mesh_method=AbaqusMeshMethod.ByNumber, solid_seeds: typing.Union[int, float] = 1,
                 vertical_pressure: float = 310,
                 predefined_stress: tuple[float, float, float, float, float, float] = (-310, -310, -310, 0, 0, 0),
                 displacement: tuple[float, float, float, float, float, float] = (0.006, 0, 0, 0, 0, 0),
                 velocity: tuple[float, float, float, float, float, float] = (1e-5, 0, 0, 0, 0, 0),
                 initial_stress_time_period: float = 1.0, shear_time_period: float = 10.0,
                 initial_stress_description: str = '', shear_description: str = '', subroutine_filepath: str = '',
                 n_state_dependent_variables: int = 0, friction_properties: tuple = None):

        super().__init__()
        self.setCalculationMethod(calculation_method)
        self.SoilGeometry(*soil_geometry)
        self.SolidGeometry(*solid_geometry)
        self.SoilMaterial(*soil_material)
        self.SolidMaterial(*solid_material)
        self.SoilMesh(soil_mesh_method, soil_seeds)
        self.SolidMesh(solid_mesh_method, solid_seeds)
        self.VerticalPressure(vertical_pressure)
        self.PredefinedConfiningStress(*predefined_stress)
        self.DefaultHistoryOutput(['U1', 'RF1'], 10)

        if calculation_method == AbaqusCalculationMethod.Standard:
            self.SoilDisplacement(*displacement)
            self.GeneralStaticStep('Initial-Stress', time_period=initial_stress_time_period,
                                   description=initial_stress_description)
            self.GeneralStaticStep('Shear', time_period=shear_time_period, description=shear_description)
            self.FrictionSubroutine(AbaqusSubroutineType.FRIC, subroutine_filepath=subroutine_filepath,
                                    n_state_dependent_variables=n_state_dependent_variables,
                                    friction_properties=friction_properties)
            self.DefaultFieldOutput(['S', 'E', 'LE', 'U', 'RF', 'RT', 'RM', 'P',
                                     'CSTRESS', 'CDISP', 'CFORCE', 'CNAREA', 'CSTATUS'])
        else:
            self.SoilVelocity(*velocity)
            self.ExplicitDynamicStep('Initial-Stress', time_period=initial_stress_time_period,
                                     description=initial_stress_description)
            self.ExplicitDynamicStep('Shear', time_period=shear_time_period, description=shear_description)
            self.FrictionSubroutine(AbaqusSubroutineType.VFRIC, subroutine_filepath=subroutine_filepath,
                                    n_state_dependent_variables=n_state_dependent_variables,
                                    friction_properties=friction_properties)
            self.DefaultFieldOutput(['S', 'E', 'LE', 'U', 'V', 'A', 'RF', 'P',
                                     'CSTRESS', 'CFORCE', 'FSLIPR', 'FSLIP', 'PPRESS'])

    def Geometry(self, name: str, length: float = 1.0, width: float = 1.0, height: float = 1.0):
        """Set geometry of the soil and solid

        Parameters
        ----------
        name: str
            name of the geometry
        length: float
            length of the soil/solid
        width: float
            width of the soil/solid
        height: float
            height of the soil/solid
        Returns
        -------
        None
        """
        if name in self.geometries.keys():
            self.geometries[name].setParameters(length, width, height)
        else:
            self.geometries[name] = AbaqusDirectShearTestGeometry(length, width, height)

    def SoilGeometry(self, length: float, width: float, height: float):
        """Set geometry of the soil, identical to Geometry('Soil', length, width, height)

        Parameters
        ----------
        length: float
            length of the soil
        width: float
            width of the soil
        height: float
            height of the soil

        Returns
        -------
        None
        """
        self.Geometry('Soil', length, width, height)

    def SolidGeometry(self, length: float, width: float, height: float):
        """Set geometry of the solid, identical to Geometry('Solid', length, width, height)

        Parameters
        ----------
        length: float
            length of the solid
        width: float
            width of the solid
        height: float
            height of the solid

        Returns
        -------
        None
        """
        self.Geometry('Solid', length, width, height)

    def SoilMaterial(self, modulus: float, poisson_ratio: float, density: float = None):
        """Set soil properties, identical to Material('Soil', modulus, poisson_ratio, density)

        Parameters
        ----------
        modulus: float
            elastic modulus of the soil
        poisson_ratio: float
            poisson's ratio of the soil, should be in the range of (0, 0.5)
        density: float
            density of the soil
        Returns
        -------
        None
        """
        self.Material('Soil', modulus, poisson_ratio, density)

    def SolidMaterial(self, modulus: float, poisson_ratio: float, density: float = None):
        """Set solid properties, identical to Material('Solid', modulus, poisson_ratio, density)

        Parameters
        ----------
        modulus: float
            elastic modulus of the solid
        poisson_ratio: float
            poisson's ratio of the solid, should be in the range of (0, 0.5)
        density: float
            density of the solid
        Returns
        -------
        None
        """
        self.Material('Solid', modulus, poisson_ratio, density)

    def VerticalPressure(self, amplitude: typing.Union[float, typing.Iterable]):
        """Set vertical pressure, identical to Pressure('Vertical-Pressure', amplitude)

        Parameters
        ----------
        amplitude: float
            amplitude of the vertical pressure

        Returns
        -------
        None
        """
        self.Pressure('Vertical-Pressure', amplitude)

    @typing.overload
    def PredefinedConfiningStress(self, predefined_stress: typing.Iterable):
        """Set predefined stress, identical to PredefinedStress('Predefined-Confining-Stress', predefined_stress)

        Parameters
        ----------
        predefined_stress: typing.Iterable
            predefined stress, an array with the dimension of 6.
        Returns
        -------
        None
        """
        pass

    @typing.overload
    def PredefinedConfiningStress(self, sig11: float, sig22: float, sig33: float, sig12: float, sig13: float,
                                  sig23: float):
        """Set predefined stress, identical to
        PredefinedStress('Predefined-Confining-Stress', sig11, sig22, sig33, sig12, sig13, sig23)

        Parameters
        ----------
        sig11: float
            component sig11 of the predefined stress
        sig22: float
            component sig22 of the predefined stress
        sig33: float
            component sig33 of the predefined stress
        sig12: float
            component sig12 of the predefined stress
        sig13: float
            component sig13 of the predefined stress
        sig23: float
            component sig23 of the predefined stress

        Returns
        -------
        None
        """
        pass

    def PredefinedConfiningStress(self, *args):
        """Set predefined stress, identical to PredefinedStress('Predefined-Confining-Stress', *args)

        Parameters
        ----------
        args: typing.Iterable or float, float, float, float, float, float
            arguments of the predefined stress, either an array with the dimension of 6, or 6 components of the predefined
            stress

        Returns
        -------
        None
        """
        self.PredefinedStress('Predefined-Confining-Stress', *args)

    @typing.overload
    def SoilDisplacement(self, displacement: typing.Iterable):
        """Set displacement of the shearing, identical to Displacement('Displacement', displacement)

        Parameters
        ----------
        displacement: typing.Iterable
            shearing displacement, an array with the dimension of 6
        Returns
        -------
        None
        """
        pass

    @typing.overload
    def SoilDisplacement(self, u1: float = 0, u2: float = 0, u3: float = 0, ur1: float = 0, ur2: float = 0,
                         ur3: float = 0):
        """Set displacement of the shearing, identical to Displacement('Displacement', u1, u2, u3, ur1, ur2, ur3)

        Parameters
        ----------
        u1: float
            component u1 of the displacement
        u2: float
            component u2 of the displacement
        u3: float
            component u3 of the displacement
        ur1: float
            component ur1 of the displacement
        ur2: float
            component ur2 of the displacement
        ur3: float
            component ur3 of the displacement

        Returns
        -------
        None
        """

        pass

    def SoilDisplacement(self, *args):
        """Set displacement of the shearing, identical to Displacement('Displacement', *args)

        Parameters
        ----------
        args: typing.Iterable or float, float, float, float, float, float
            arguments of the displacement, either an array with the dimension of 6, or 6 components of the displacement

        Returns
        -------
        None
        """
        self.Displacement('Displacement', *args)

    @typing.overload
    def SoilVelocity(self, velocity: typing.Iterable):
        """Set velocity of the shearing, identical to Velocity('Velocity', velocity)

        Parameters
        ----------
        velocity: typing.Iterable
            shearing velocity, an array with the dimension of 6
        Returns
        -------
        None
        """
        pass

    @typing.overload
    def SoilVelocity(self, v1: float = 0, v2: float = 0, v3: float = 0, vr1: float = 0, vr2: float = 0,
                     vr3: float = 0):
        """Set velocity of the shearing, identical to Velocity('Velocity', v1, v2, v3, vr1, vr2, vr3)

        Parameters
        ----------
        v1: float
            component v1 of the velocity
        v2: float
            component v2 of the velocity
        v3: float
            component v3 of the velocity
        vr1: float
            component vr1 of the velocity
        vr2: float
            component vr2 of the velocity
        vr3: float
            component vr3 of the velocity

        Returns
        -------
        None
        """
        pass

    def SoilVelocity(self, *args):
        """Set velocity of the shearing, identical to Velocity('Velocity', *args)

        Parameters
        ----------
        args: typing.Iterable or float, float, float, float, float, float
            arguments of the displacement, either an array with the dimension of 6, or 6 components of the velocity

        Returns
        -------
        None
        """
        self.Velocity('Velocity', *args)

    @typing.overload
    def SoilMesh(self, method: AbaqusMeshMethod.ByNumber, seeds_number: int):
        """Set mesh parameter of the soil by number of seeds, identical to Mesh('Soil', method, seeds_number)

        Parameters
        ----------
        method: AbaqusMeshMethod
            mesh method, should be AbaqusMeshMethod.ByNumber
        seeds_number: int
            number of seeds in each edge
        Returns
        -------
        None
        """
        pass

    @typing.overload
    def SoilMesh(self, method: AbaqusMeshMethod.BySize, seeds_size: float):
        """Set mesh parameter of the soil by size of seeds, identical to Mesh('Soil', method, seeds_size)

        Parameters
        ----------
        method: AbaqusMeshMethod
            mesh method, should be AbaqusMeshMethod.BySize
        seeds_size: float
            spacing size of seeds in each edge
        Returns
        -------
        None
        """
        pass

    def SoilMesh(self, method: AbaqusMeshMethod, seeds: typing.Union[int, float]):
        """Set mesh parameter of the soil by number/size of seeds, identical to Mesh('Soil', method, seeds)

        Parameters
        ----------
        method: AbaqusMeshMethod
            mesh method, should be AbaqusMeshMethod.BySize
        seeds: float or int
            number of seeds or spacing size of seeds in each edge
        Returns
        -------
        None
        """
        self.Mesh('Soil', method, seeds)

    @typing.overload
    def SolidMesh(self, method: AbaqusMeshMethod.ByNumber, seeds_number: int):
        """Set mesh parameter of the solid by number of seeds, identical to Mesh('Solid', method, seeds_number)

        Parameters
        ----------
        method: AbaqusMeshMethod
            mesh method, should be AbaqusMeshMethod.ByNumber
        seeds_number: int
            number of seeds in each edge
        Returns
        -------
        None
        """
        pass

    @typing.overload
    def SolidMesh(self, method: AbaqusMeshMethod.BySize, seeds_size: float):
        """Set mesh parameter of the solid by size of seeds, identical to Mesh('Solid', method, seeds_size)

        Parameters
        ----------
        method: AbaqusMeshMethod
            mesh method, should be AbaqusMeshMethod.BySize
        seeds_size: float
            spacing size of seeds in each edge
        Returns
        -------
        None
        """
        pass

    def SolidMesh(self, method: AbaqusMeshMethod, seeds: typing.Union[int, float]):
        """Set mesh parameter of the solid by number/size of seeds, identical to Mesh('Solid', method, seeds)

        Parameters
        ----------
        method: AbaqusMeshMethod
            mesh method, should be AbaqusMeshMethod.BySize
        seeds: float or int
            number of seeds or spacing size of seeds in each edge
        Returns
        -------
        None
        """
        self.Mesh('Solid', method, seeds)

    @typing.overload
    def FrictionSubroutine(self, subroutine_type=AbaqusSubroutineType.FRIC, subroutine_filepath: str = '',
                           n_state_dependent_variables: int = 0, cohesion: float = 0.0, G0: float = 250.0,
                           R: float = 5.19, e_cref: float = 0.89, Lambda: float = 0.147, xi: float = 0.424,
                           phi: float = 31.2, dd: float = 7.57, n_p: float = 2.06, n_d: float = 0.46,
                           e0: float = 0.5771, thickness: float = 5 * 0.23 * 1e-3):
        """Set properties of the exponential interface model (FRIC/VFRIC) subroutine,

        Common properties

        - Dense: [0.0,  250.0, 5.19, 0.89, 0.147, 0.424, 31.2, 7.57, 2.06,  0.46, 0.5771, 0.00115]
        - Loose: [0.0,  290.0, 2.69, 1.20, 0.750, 0.161, 29.5, 14.2, 0.10,  0.10, 0.7700, 0.00115]
        - C:     [0.0, 2000.0, 6.96, 0.90, 0.413, 1.000, 36.4, 0.10, 5.70, 14.01, 0.5771, 0.00500]

        Parameters
        ----------
        subroutine_type: AbaqusSubroutineType
            subroutine type, either AbaqusSubroutineType.FRIC or AbaqusSubroutineType.VFRIC
        subroutine_filepath: str
            filepath of the subroutine
        n_state_dependent_variables: int
            number of dependent variables
        cohesion: float
            cohesion of the soil
        G0: float
            initial shear modulus
        R: float
            a material constant
        e_cref: float
            reference void ratio
        Lambda: float
            parameter controlling the shape of CSL line
        xi: float
            parameter controlling the nonlinearity of CSL line
        phi: float
            friction angle
        dd: float
            a dilatancy parameter
        n_p: float
            a parameter in the critical state concept
        n_d: float
            a parameter in the critical state concept
        e0: float
            initial void ratio
        thickness: float
            thickness of the soil-structure interface

        Returns
        -------
        None
        """
        pass

    @typing.overload
    def FrictionSubroutine(self, subroutine_type=AbaqusSubroutineType.FRIC, subroutine_filepath: str = '',
                           n_state_dependent_variables: int = 0, friction_properties: typing.Iterable = None):
        """Set properties of the FRIC/VFRIC subroutine

        Parameters
        ----------
        subroutine_type: AbaqusSubroutineType
            subroutine type, either AbaqusSubroutineType.FRIC or AbaqusSubroutineType.VFRIC
        subroutine_filepath: str
            filepath of the subroutine
        n_state_dependent_variables: int
            number of dependent variables
        friction_properties: typing.Iterable
            friction properties in the FRIC/VFRIC subroutine
        Returns
        -------
        None
        """
        pass

    def FrictionSubroutine(self, subroutine_type=AbaqusSubroutineType.FRIC, subroutine_filepath: str = '',
                           n_state_dependent_variables: int = 0, *args, **kwargs):
        """Set properties of the FRIC/VFRIC subroutine

        Parameters
        ----------
        subroutine_type: AbaqusSubroutineType
            subroutine type, either AbaqusSubroutineType.FRIC or AbaqusSubroutineType.VFRIC
        subroutine_filepath: str
            filepath of the subroutine
        n_state_dependent_variables: int
            number of dependent variables
        args, kwargs
            friction properties in the FRIC/VFRIC subroutine, if you use non-key-value way to call this function, please
            provide in correct order, if you use the key-value way to call this function, please provide all the
            properties, if you call this function in a mixed way, please provide the properties in correct order in
            non-key-value and it must be the first several properties, please provide the rest properties in key-value
            way. Whatever in what way you use, all parameters must be specified, otherwise an error would occur.
        Returns
        -------
        None
        """
        if len(args) + len(kwargs) == 1:
            friction_properties = args[0] if len(args) == 1 else kwargs['friction_properties']
        else:
            keys = ['cohesion', 'G0', 'R', 'e_cref', 'Lambda', 'xi', 'phi', 'dd', 'n_p', 'n_d', 'e0', 'thickness']
            for arg, key in zip(args, keys[:len(args)]):
                kwargs[key] = arg
            friction_properties = [kwargs[key] for key in keys]
        self.Subroutine('Friction', subroutine_type, subroutine_filepath, n_state_dependent_variables,
                        friction_properties)

    @staticmethod
    def _setParameters(text: str, **kwargs):
        for key, value in kwargs.items():
            pattern = '\n{}\s?=\s?.+\n'.format(key)
            if type(value) != str:
                text = re.sub(pattern, '\n{} = {}\n'.format(key, value), text)
            else:
                text = re.sub(pattern, "\n{} = '{}'\n".format(key, value), text)
        return text

    def generateAbaqusPythonScript(self):
        """Generate abaqus-python script file to the work directory

        Returns
        -------
        None
        """
        # TODO ABAQUS MAIN MODEL SCRIPT
        output_template_name = 'dst.py'
        template_path = os.path.join(os.path.split(os.path.abspath(__file__))[0], output_template_name)
        with open(template_path, 'r+') as file:
            template_text = file.read()

        template_text = self._setParameters(
            template_text,
            CALCULATION_METHOD=self.calculationMethodString(),
            SOIL_LENGTH=self.geometries['Soil'].length,  # SOIL GEOMETRY
            SOIL_WIDTH=self.geometries['Soil'].width,
            SOIL_HEIGHT=self.geometries['Soil'].height,
            SOLID_LENGTH=self.geometries['Solid'].length,  # SOLID GEOMETRY
            SOLID_WIDTH=self.geometries['Solid'].width,
            SOLID_HEIGHT=self.geometries['Solid'].height,
            SOIL_ELASTIC_MODULUS=self.materials['Soil'].elastic.modulus,  # SOIL MATERIAL PROPERTIES
            SOIL_POISSON_RATIO=self.materials['Soil'].elastic.poisson_ratio,
            SOIL_DENSITY=self.materials['Soil'].density,
            SOLID_ELASTIC_MODULUS=self.materials['Solid'].elastic.modulus,  # SOLID MATERIAL PROPERTIES
            SOLID_POISSON_RATIO=self.materials['Solid'].elastic.poisson_ratio,
            SOLID_DENSITY=self.materials['Solid'].density,
            SOIL_MESH_METHOD=self.meshes['Soil'].methodString(),  # MESH PARAMETERS
            SOIL_SEEDS_NUMBER=self.meshes['Soil'].seeds_number,
            SOIL_SEEDS_SIZE=self.meshes['Soil'].seeds_size,
            SOLID_MESH_METHOD=self.meshes['Solid'].methodString(),
            SOLID_SEEDS_NUMBER=self.meshes['Solid'].seeds_number,
            SOLID_SEEDS_SIZE=self.meshes['Solid'].seeds_size,
            VERTICAL_PRESSURE=self.loads['Vertical-Pressure'].amplitude,  # VERTICAL PRESSURE
            PREDEFINED_STRESS=self.loads['Predefined-Confining-Stress'].predefined_stress,  # PREDEFINED STRESS
            SUBROUTINE=self.subroutines['Friction'].subroutineString(),  # FRIC/VFRIC SUBROUTINE
            SUBROUTINE_PATH=self.subroutines['Friction'].subroutine_filepath,
            N_STATE_DEPENDENT_VARS=self.subroutines['Friction'].n_state_dependent_variables,
            FRICTION_PARAMETERS=self.subroutines['Friction'].properties,
            FIELD_OUTPUT_VARIABLES=self.outputs['F-Output-1'].variables,
            HISTORY_OUTPUT_VARIABLES=self.outputs['H-Output-1'].variables, 
            EXPLICIT_NUM_INTERVALS=self.outputs['H-Output-1'].num_intervals,  # FIXME
        )
        if self.calculation_method == AbaqusCalculationMethod.Standard:
            template_text = self._setParameters(
                template_text, RP_DISPLACEMENT=self.loads['Displacement'].displacement,
                INITIAL_STRESS_TIME_PERIOD=self.steps['Initial-Stress'].time_period,
                INITIAL_STRESS_DESCRIPTION=self.steps['Initial-Stress'].description,
                INITIAL_STRESS_INCREMENTATION_TYPE=self.steps['Initial-Stress'].incrementation.incrementationType(),
                INITIAL_STRESS_MAXIMAL_INCREMENTS=self.steps['Initial-Stress'].incrementation.maximal_increments,
                INITIAL_STRESS_INITIAL_INCREMENT_SIZE=self.steps['Initial-Stress'].incrementation.initial_increment_size,
                INITIAL_STRESS_MINIMAL_INCREMENT_SIZE=self.steps['Initial-Stress'].incrementation.minimal_increment_size,
                INITIAL_STRESS_MAXIMAL_INCREMENT_SIZE=self.steps['Initial-Stress'].incrementation.maximal_increment_size,
                INITIAL_STRESS_FIXED_INCREMENT_SIZE=self.steps['Initial-Stress'].incrementation.fixed_increment_size,
                SHEAR_TIME_PERIOD=self.steps['Shear'].time_period,
                SHEAR_DESCRIPTION=self.steps['Shear'].description,
                SHEAR_INCREMENTATION_TYPE=self.steps['Shear'].incrementation.incrementationType(),
                SHEAR_MAXIMAL_INCREMENTS=self.steps['Shear'].incrementation.maximal_increments,
                SHEAR_INITIAL_INCREMENT_SIZE=self.steps['Shear'].incrementation.initial_increment_size,
                SHEAR_MINIMAL_INCREMENT_SIZE=self.steps['Shear'].incrementation.minimal_increment_size,
                SHEAR_MAXIMAL_INCREMENT_SIZE=self.steps['Shear'].incrementation.maximal_increment_size,
                SHEAR_FIXED_INCREMENT_SIZE=self.steps['Shear'].incrementation.fixed_increment_size,
            )
        else:
            template_text = self._setParameters(
                template_text, RP_VELOCITY=self.loads['Velocity'].velocity,
                INITIAL_STRESS_TIME_PERIOD=self.steps['Initial-Stress'].time_period,
                INITIAL_STRESS_DESCRIPTION=self.steps['Initial-Stress'].description,
                INITIAL_STRESS_MAXIMAL_TIME_INCREMENT=self.steps['Initial-Stress'].incrementation.maximal_time_increment,
                INITIAL_STRESS_INCREMENT_SIZE=self.steps['Initial-Stress'].incrementation.increment_size,
                INITIAL_STRESS_SCALING_FACTOR=self.steps['Initial-Stress'].incrementation.scaling_factor,
                INITIAL_STRESS_LINEAR_BULK_VISCOSITY=self.steps['Initial-Stress'].linear_bulk_viscosity,
                INITIAL_STRESS_QUAD_BULK_VISCOSITY=self.steps['Initial-Stress'].quad_bulk_viscosity,
                SHEAR_TIME_PERIOD=self.steps['Shear'].time_period,
                SHEAR_DESCRIPTION=self.steps['Shear'].description,
                SHEAR_MAXIMAL_TIME_INCREMENT=self.steps['Shear'].incrementation.maximal_time_increment,
                SHEAR_INCREMENT_SIZE=self.steps['Shear'].incrementation.increment_size,
                SHEAR_SCALING_FACTOR=self.steps['Shear'].incrementation.scaling_factor,
                SHEAR_LINEAR_BULK_VISCOSITY=self.steps['Shear'].linear_bulk_viscosity,
                SHEAR_QUAD_BULK_VISCOSITY=self.steps['Shear'].quad_bulk_viscosity
            )
        template_text = template_text.replace('new directory', self.work_directory)
        pattern = '\n{}.+\n'.format('WORK_DIRECTORY\s?=\s?')
        template_text = re.sub(pattern, '\n{}\n'.format("WORK_DIRECTORY = r'new directory'"), template_text)
        template_text = template_text.replace('new directory', self.work_directory)
        template_text = template_text.replace('dst.cae', self.model_name + '.cae')

        with open(os.path.join(self.work_directory, self.model_name + '.py'), 'w+') as file:
            file.write(template_text)

        # TODO ABAQUS OUTPUT SCRIPT FILE
        if self.calculation_method == AbaqusCalculationMethod.Standard:
            output_template_name = 'dst_output_fric.py'
        else:
            output_template_name = 'dst_output_vfric.py'
        output_template_path = os.path.join(os.path.split(os.path.abspath(__file__))[0], output_template_name)
        with open(output_template_path, 'r+') as file:
            output_template_text = file.read()

        output_template_text = output_template_text.replace('new directory', self.work_directory)
        pattern = '\n{}.+\n'.format('WORK_DIRECTORY\s?=\s?')
        output_template_text = re.sub(pattern, '\n{}\n'.format("WORK_DIRECTORY = r'new directory'"), output_template_text)
        output_template_text = output_template_text.replace('new directory', self.work_directory)
        output_template_text = output_template_text.replace('dst.cae', self.model_name + '.cae')

        output_template_text = self._setParameters(
            output_template_text,
            SUBROUTINE=self.subroutines['Friction'].subroutineString(),
        )

        with open(os.path.join(self.work_directory, self.output_name + '.py'), 'w+') as file:
            file.write(output_template_text)

        try:
            shutil.copy(os.path.join(self.old_work_directory, self.subroutines['Friction'].subroutine_filepath),
                        self.work_directory)
        except FileNotFoundError:
            pass
        except shutil.SameFileError:
            pass

    def submit(self):
        """Submit the abaqus job, 3 steps will be done in this function

        - use abaqus command to run the model script file the build the abaqus model
        - use abaqus command to submit the job
        - use abaqus command to ru the output script file to obtain the output data and save it to the work directory

        Returns
        -------

        """
        os.system('abaqus cae noGUI={}.py'.format(self.model_name))
        os.system('abaqus job=Job-1 user={} cpus=1 int double'.format(self.subroutines['Friction'].subroutine_filepath))
        os.system('abaqus cae noGUI={}.py'.format(self.output_name))

        for filename in os.listdir():
            if os.path.splitext(filename)[-1] in ['.com', '.dat', '.log', '.msg', '.prt', '.rpy', '.sim', '.sta', '.1']:
                os.remove(filename)

    def plot(self):
        """Plot some figures when everything has been done properly, more specifically, the output data have been saved
        to the DATA folder in the work directory

        Returns
        -------

        """
        os.chdir(self.work_directory)

        friction_parameters = (parameter[0] for parameter in self.subroutines['Friction'].properties)
        (cohesion, G0, R, e_ref0, Lambda, xi, phi_u, dd, n_p, n_d, e0, thickness) = friction_parameters

        if self.calculation_method == AbaqusCalculationMethod.Standard:
            # TODO LOAD FRIC DATA
            GAMMA_NORM = np.loadtxt('DATA/GAMMA_NORM.csv', skiprows=1, delimiter=',')
            CSHEAR1 = np.loadtxt('DATA/CSHEAR1.csv', skiprows=1, delimiter=',')
            CSLIP1 = np.loadtxt('DATA/CSLIP1.csv', skiprows=1, delimiter=',')
            U1_RP = np.loadtxt('DATA/U1_RP.csv', skiprows=1, delimiter=',')
            EPSN_FRIC = np.loadtxt('DATA/EPSN_FRIC.csv', skiprows=1, delimiter=',')
            U3_TOP = np.loadtxt('DATA/U3_TOP.csv', skiprows=1, delimiter=',')
            RF1_RP = np.loadtxt('DATA/RF1_RP.csv', skiprows=1, delimiter=',')

            FRIC_TANGENTIAL_DISPLACEMENT = GAMMA_NORM[:, 1] * thickness * 1e3
            FRIC_SHEAR_STRESS = CSHEAR1[:, 1]
            FRIC_U1_RP = abs(U1_RP[:, 1]) * 1e3
            FRIC_CSLIP1 = abs(CSLIP1[:, 1]) * 1e3
            FRIC_RF1_RP = RF1_RP[:, 1] / (self.geometries['Soil'].length * self.geometries['Soil'].width)

            FRIC_EPSN = EPSN_FRIC[:, 1] * thickness * 1e3 * (-1)
            FRIC_UTOP = (U3_TOP[:, 1] - U3_TOP[0, 1]) * 1e3

            WORK_DIRECTORY = os.getcwd()
            if not os.path.exists(os.path.join(WORK_DIRECTORY, 'Figures')):
                os.mkdir(os.path.join(WORK_DIRECTORY, 'Figures'))

            plt.figure(figsize=(8, 5))
            plt.plot(FRIC_TANGENTIAL_DISPLACEMENT, FRIC_SHEAR_STRESS, 'c-', label='FRIC (GAMMA_N - CSHEAR1)')
            plt.plot(FRIC_U1_RP, FRIC_SHEAR_STRESS, 'r:', label='FRIC (RPU1 - CSHEAR1)')
            plt.plot(FRIC_U1_RP, FRIC_RF1_RP, 'k--', label='FRIC (RPU1 - RPRF1)')
            plt.plot(FRIC_CSLIP1, FRIC_SHEAR_STRESS, 'm-', label='FRIC (CSLIP1 - CSHEAR1)')
            plt.xlabel('Tangential displacement (mm)')
            plt.ylabel('Shear stress (kPa)')
            plt.legend()
            plt.grid()
            plt.tight_layout()
            plt.savefig('Figures/TANGENTIAL-DISPLACEMENT-SHEAR-STRESS.pdf')

            plt.figure(figsize=(8, 5))
            plt.plot(FRIC_TANGENTIAL_DISPLACEMENT, FRIC_EPSN, 'c-', label='FRIC (GAMMA_N - EPSN)')
            plt.plot(FRIC_U1_RP, FRIC_EPSN, 'r:', label='FRIC (RPU1 - EPSN)')
            plt.plot(FRIC_U1_RP, FRIC_UTOP, 'm-', label='FRIC (RPU1 - UTOP)')
            plt.plot(FRIC_CSLIP1, FRIC_EPSN, 'k--', label='FRIC (CSLIP1 - EPSN)')
            plt.xlabel('Tangential displacement (mm)')
            plt.ylabel('Normal displacement (mm)')
            plt.legend()
            plt.grid()
            plt.tight_layout()
            plt.savefig('Figures/TANGENTIAL-DISPLACEMENT-NORMAL-DISPLACEMENT.pdf')
        else:
            # TODO LOAD VFRIC DATA
            # GAMMA_NORM = np.loadtxt('DATA/GAMMA_NORM.csv', skiprows=1, delimiter=',')
            CSHEAR1 = np.loadtxt('DATA/CSHEAR1.csv', skiprows=1, delimiter=',')
            # CSLIP1 = np.loadtxt('DATA/CSLIP1.csv', skiprows=1, delimiter=',')
            U1_RP = np.loadtxt('DATA/U1_RP.csv', skiprows=1, delimiter=',')
            # EPSN_FRIC = np.loadtxt('DATA/EPSN_FRIC.csv', skiprows=1, delimiter=',')
            # U3_TOP = np.loadtxt('DATA/U3_TOP.csv', skiprows=1, delimiter=',')
            RF1_RP = np.loadtxt('DATA/RF1_RP.csv', skiprows=1, delimiter=',')
            FSLIPEQ = np.loadtxt('DATA/FSLIPEQ.csv', skiprows=1, delimiter=',')

            # VFRIC_TANGENTIAL_DISPLACEMENT = GAMMA_NORM[:, 1] * thickness * 1e3
            VFRIC_SHEAR_STRESS = CSHEAR1[:, 1]
            VFRIC_U1_RP = abs(U1_RP[:, 1]) * 1e3
            # VFRIC_CSLIP1 = abs(CSLIP1[:, 1]) * 1e3
            VFRIC_RF1_RP = RF1_RP[:, 1] / (self.geometries['Soil'].length * self.geometries['Soil'].width)
            VFRIC_FSLIPEQ = FSLIPEQ[:, 1] * 1000

            # VFRIC_EPSN = EPSN_FRIC[:, 1] * thickness * 1e3 * (-1)
            # VFRIC_UTOP = (U3_TOP[:, 1] - U3_TOP[0, 1]) * 1e3

            WORK_DIRECTORY = os.getcwd()
            if not os.path.exists(os.path.join(WORK_DIRECTORY, 'Figures')):
                os.mkdir(os.path.join(WORK_DIRECTORY, 'Figures'))

            plt.figure(figsize=(8, 5))
            # plt.plot(FRIC_TANGENTIAL_DISPLACEMENT, VFRIC_SHEAR_STRESS, 'c-', label='VFRIC (GAMMA_N - CSHEAR1)')
            plt.plot(VFRIC_U1_RP, VFRIC_SHEAR_STRESS, 'r:', label='VFRIC (RPU1 - CSHEAR1)')
            plt.plot(VFRIC_U1_RP, VFRIC_RF1_RP, 'k--', label='VFRIC (RPU1 - RPRF1)')
            plt.plot(VFRIC_FSLIPEQ, VFRIC_SHEAR_STRESS, 'm-', label='VFRIC (FSLIPQ - CSHEAR1)')
            # plt.plot(FRIC_CSLIP1, VFRIC_SHEAR_STRESS, 'm-', label='VFRIC (CSLIP1 - CSHEAR1)')
            plt.xlabel('Tangential displacement (mm)')
            plt.ylabel('Shear stress (kPa)')
            plt.legend()
            plt.grid()
            plt.tight_layout()
            plt.savefig('Figures/TANGENTIAL-DISPLACEMENT-SHEAR-STRESS.pdf')

            plt.figure(figsize=(8, 5))
            # plt.plot(VFRIC_TANGENTIAL_DISPLACEMENT, VFRIC_EPSN, 'c-', label='FRIC (GAMMA_N - EPSN)')
            # plt.plot(VFRIC_U1_RP, VFRIC_EPSN, 'r:', label='VFRIC (RPU1 - EPSN)')
            # plt.plot(VFRIC_FSLIPQ, VFRIC_EPSN, 'r:', label='VFRIC (RPU1 - EPSN)')
            # plt.plot(VFRIC_U1_RP, VFRIC_UTOP, 'm-', label='VFRIC (RPU1 - UTOP)')
            # plt.plot(VFRIC_CSLIP1, VFRIC_EPSN, 'k--', label='VFRIC (CSLIP1 - EPSN)')
            plt.xlabel('Tangential displacement (mm)')
            plt.ylabel('Normal displacement (mm)')
            plt.legend()
            plt.grid()
            plt.tight_layout()
            # plt.savefig('Figures/TANGENTIAL-DISPLACEMENT-NORMAL-DISPLACEMENT.pdf')

            # plt.show()



import typing

from .AbaqusVariable import AbaqusVariable


class AbaqusLoad(AbaqusVariable):
    pass


class AbaqusPressure(AbaqusLoad):
    amplitude: typing.Union[float, typing.Iterable] = None

    def __init__(self, amplitude: typing.Union[float, typing.Iterable] = None):
        self.amplitude = amplitude

    def setParameters(self, amplitude: typing.Union[float, typing.Iterable] = None):
        self.amplitude = amplitude


class AbaqusPredefinedStress(AbaqusLoad):
    predefined_stress: typing.Iterable = None

    @typing.overload
    def __init__(self, predefined_stress: typing.Iterable = None):
        pass

    @typing.overload
    def __init__(self, sig11: float, sig22: float, sig33: float, sig12: float, sig13: float, sig23: float):
        pass

    def __init__(self, *args):
        self.setParameters(*args)

    @typing.overload
    def setParameters(self, predefined_stress: typing.Iterable):
        pass

    @typing.overload
    def setParameters(self, sig11: float, sig22: float, sig33: float, sig12: float, sig13: float, sig23: float):
        pass

    def setParameters(self, *args):
        if len(args) == 6:
            self.predefined_stress = [args[0], args[1], args[2], args[3], args[4], args[5]]
        else:
            self.predefined_stress = args[0]


class AbaqusDisplacement(AbaqusLoad):
    displacement: typing.Iterable = None

    @typing.overload
    def __init__(self, displacement: typing.Iterable = None):
        pass

    @typing.overload
    def __init__(self, u1: float = 0, u2: float = 0, u3: float = 0, ur1: float = 0, ur2: float = 0, ur3: float = 0):
        pass

    def __init__(self, *args):
        self.setParameters(*args)

    @typing.overload
    def setParameters(self, displacement: typing.Iterable):
        pass

    @typing.overload
    def setParameters(self, u1: float = 0, u2: float = 0, u3: float = 0, ur1: float = 0, ur2: float = 0, ur3: float = 0):
        pass

    def setParameters(self, *args):
        if len(args) == 6:
            self.displacement = [args[0], args[1], args[2], args[3], args[4], args[5]]
        else:
            self.displacement = args[0]


class AbaqusVelocity(AbaqusLoad):
    velocity: typing.Iterable = None

    @typing.overload
    def __init__(self, velocity: typing.Iterable = None):
        pass

    @typing.overload
    def __init__(self, v1: float = 0, v2: float = 0, v3: float = 0, vr1: float = 0, vr2: float = 0, vr3: float = 0):
        pass

    def __init__(self, *args):
        self.setParameters(*args)

    @typing.overload
    def setParameters(self, velocity: typing.Iterable):
        pass

    @typing.overload
    def setParameters(self, v1: float = 0, v2: float = 0, v3: float = 0, vr1: float = 0, vr2: float = 0, vr3: float = 0):
        pass

    def setParameters(self, *args):
        if len(args) == 6:
            self.velocity = [args[0], args[1], args[2], args[3], args[4], args[5]]
        else:
            self.velocity = args[0]





import typing
from enum import IntEnum

from .AbaqusVariable import AbaqusVariable


class AbaqusCalculationMethod(IntEnum):
    Standard = 0
    Explicit = 1


class AbaqusBaseStepIncrementationType(IntEnum):
    Automatic = 0
    Fixed = 1


class AbaqusBaseIncrementation(AbaqusVariable):
    incrementation_type = AbaqusBaseStepIncrementationType.Automatic

    def incrementationType(self):
        if self.incrementation_type == AbaqusBaseStepIncrementationType.Automatic:
            return 'AUTOMATIC'
        else:
            return 'FIXED'


class AbaqusStep(AbaqusVariable):
    calculation_method = AbaqusCalculationMethod.Standard
    time_period: float = 1
    description: str = ''

    def __init__(self, calculation_method=AbaqusCalculationMethod.Standard, time_period: float = 1,
                 description: str = ''):
        self.calculation_method = calculation_method
        self.time_period, self.description = time_period, description

    def setParameters(self, calculation_method: AbaqusCalculationMethod, time_period: float, description: str):
        self.calculation_method = calculation_method
        self.time_period, self.description = time_period, description


class AbaqusGeneralStaticStepIncrementation(AbaqusBaseIncrementation):
    # incrementation_type = AbaqusBaseStepIncrementationType.Automatic
    maximal_increments: int = 100
    initial_increment_size: float = 1.0
    minimal_increment_size: float = 1e-5
    maximal_increment_size: float = 1.0
    fixed_increment_size: float = None

    def __init__(self, incrementation_type=AbaqusBaseStepIncrementationType.Automatic, maximal_increments: int = 100,
                 initial_increment_size: float = 1.0, minimal_increment_size: float = 1e-5,
                 maximal_increment_size: float = 1.0, fixed_increment_size: float = None):
        if incrementation_type == AbaqusBaseStepIncrementationType.Automatic:
            self.setParameters(incrementation_type, maximal_increments, initial_increment_size, minimal_increment_size,
                               maximal_increment_size)
        elif incrementation_type == AbaqusBaseStepIncrementationType.Fixed:
            self.setParameters(incrementation_type, maximal_increments, fixed_increment_size)

    @typing.overload
    def setParameters(self, incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                      maximal_increments: int = 100, initial_increment_size: float = 1.0,
                      minimal_increment_size: float = 1e-5, maximal_increment_size: float = 1.0):
        pass

    @typing.overload
    def setParameters(self, incrementation_type=AbaqusBaseStepIncrementationType.Fixed,
                      maximal_increments: int = 100, fixed_increment_size: float = 0.1):
        pass

    def setParameters(self, incrementation_type=AbaqusBaseStepIncrementationType.Automatic, *args, **kwargs):
        if incrementation_type == AbaqusBaseStepIncrementationType.Fixed:
            keys = ['maximal_increments', 'fixed_increment_size']
            for arg, key in zip(args, keys[:len(args)]):
                kwargs[key] = arg
            maximal_increments = kwargs['maximal_increments'] if 'maximal_increments' in kwargs.keys() else 100
            fixed_increment_size = kwargs['fixed_increment_size'] if 'fixed_increment_size' in kwargs.keys() else 0.1
            self.incrementation_type = incrementation_type
            self.maximal_increments, self.fixed_increment_size = maximal_increments, fixed_increment_size
        else:
            keys = ['maximal_increments', 'initial_increment_size', 'minimal_increment_size', 'maximal_increment_size']
            for arg, key in zip(args, keys[:len(args)]):
                kwargs[key] = arg
            maximal_increments = kwargs['maximal_increments'] if 'maximal_increments' in kwargs.keys() else 100
            initial_increment_size = kwargs['initial_increment_size'] if 'initial_increment_size' in kwargs.keys() else 1.0
            minimal_increment_size = kwargs['minimal_increment_size'] if 'minimal_increment_size' in kwargs.keys() else 1e-5
            maximal_increment_size = kwargs['maximal_increment_size'] if 'maximal_increment_size' in kwargs.keys() else 1.0
            self.incrementation_type = incrementation_type
            self.maximal_increments, self.initial_increment_size = maximal_increments, initial_increment_size
            self.minimal_increment_size, self.maximal_increment_size = minimal_increment_size, maximal_increment_size


class AbaqusGeneralStaticStep(AbaqusStep):
    incrementation: AbaqusGeneralStaticStepIncrementation

    def __init__(self, time_period: float = 1.0, description: str = '',
                 incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                 maximal_increments: int = 100, initial_increment_size: float = 1.0,
                 minimal_increment_size: float = 1e-5, maximal_increment_size: float = 1.0,
                 fixed_increment_size: float = None):
        super(AbaqusGeneralStaticStep, self).__init__(AbaqusCalculationMethod.Standard, time_period, description)
        self.incrementation = AbaqusGeneralStaticStepIncrementation()
        if incrementation_type == AbaqusBaseStepIncrementationType.Automatic:
            self.incrementation.setParameters(incrementation_type, maximal_increments, initial_increment_size,
                                              minimal_increment_size, maximal_increment_size)
        elif incrementation_type == AbaqusBaseStepIncrementationType.Fixed:
            self.incrementation.setParameters(incrementation_type, maximal_increments, fixed_increment_size)

    @typing.overload
    def setParameters(self, time_period: float, description: str,
                      incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                      maximal_increments: int = 100, initial_increment_size: float = 0.1,
                      minimal_increment_size: float = 1e-5, maximal_increment_size: float = 1.0):
        pass

    @typing.overload
    def setParameters(self, time_period: float, description: str,
                      incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                      maximal_increments: int = 100, fixed_increment_size: float = 0.1):
        pass

    def setParameters(self, time_period: float, description: str,
                      incrementation_type=AbaqusBaseStepIncrementationType.Automatic, *args, **kwargs):
        super(AbaqusGeneralStaticStep, self).setParameters(AbaqusCalculationMethod.Standard, time_period, description)
        self.incrementation.setParameters(incrementation_type, *args, **kwargs)


class AbaqusExplicitDynamicStepIncrementation(AbaqusBaseIncrementation):
    # incrementation_type = AbaqusBaseStepIncrementationType.Automatic
    maximal_time_increment: typing.Union[float, str] = 'UNLIMITED'
    increment_size: typing.Union[float, str] = 'ELEMENT_BY_ELEMENT_INCREMENTATION'
    scaling_factor: float = 1.0

    def __init__(self, scaling_factor: float = 1.0, incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                 maximal_time_increment: typing.Union[float, str] = 'UNLIMITED',
                 increment_size: typing.Union[float, str] = 'ELEMENT_BY_ELEMENT_INCREMENTATION'):
        if incrementation_type == AbaqusBaseStepIncrementationType.Automatic:
            self.setParameters(scaling_factor, incrementation_type, maximal_time_increment)
        elif incrementation_type == AbaqusBaseStepIncrementationType.Fixed:
            self.setParameters(scaling_factor, incrementation_type, increment_size)

    @typing.overload
    def setParameters(self, scaling_factor: float = 1.0, incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                      maximal_time_increment: typing.Union[float, str] = 'UNLIMITED'):
        pass

    @typing.overload
    def setParameters(self, scaling_factor: float = 1.0, incrementation_type=AbaqusBaseStepIncrementationType.Fixed,
                      increment_size: typing.Union[float, str] = 'ELEMENT_BY_ELEMENT_INCREMENTATION'):
        pass

    def setParameters(self, scaling_factor: float = 1.0, incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                      *args, **kwargs):
        self.scaling_factor = scaling_factor
        if incrementation_type == AbaqusBaseStepIncrementationType.Automatic:
            self.incrementation_type = incrementation_type
            self.maximal_time_increment = args[0] if len(args) > 0 else \
                kwargs['maximal_time_increment'] if 'maximal_time_increment' in kwargs.keys() else 'UNLIMITED'
        elif incrementation_type == AbaqusBaseStepIncrementationType.Fixed:
            self.incrementation_type = incrementation_type
            self.increment_size = args[0] if len(args) > 0 else \
                kwargs['increment_size'] if 'increment_size' in kwargs.keys() else 'ELEMENT_BY_ELEMENT_INCREMENTATION'


class AbaqusExplicitDynamicStep(AbaqusStep):
    linear_bulk_viscosity: float = 0.06
    quad_bulk_viscosity: float = 1.2

    incrementation: AbaqusExplicitDynamicStepIncrementation

    def __init__(self, time_period: float, description: str,
                 linear_bulk_viscosity: float, quad_bulk_viscosity: float, scaling_factor: float = 1.0,
                 incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                 maximal_time_increment: typing.Union[float, str] = 'UNLIMITED',
                 increment_size: typing.Union[float, str] = 'ELEMENT_BY_ELEMENT_INCREMENTATION'):
        super(AbaqusExplicitDynamicStep, self).__init__(AbaqusCalculationMethod.Explicit, time_period, description)
        self.incrementation = AbaqusExplicitDynamicStepIncrementation()
        if incrementation_type == AbaqusBaseStepIncrementationType.Automatic:
            self.setParameters(time_period, description, linear_bulk_viscosity,
                               quad_bulk_viscosity, scaling_factor, incrementation_type, maximal_time_increment)
        elif incrementation_type == AbaqusBaseStepIncrementationType.Fixed:
            self.setParameters(time_period, description, linear_bulk_viscosity,
                               quad_bulk_viscosity, scaling_factor, incrementation_type, increment_size)

    @typing.overload
    def setParameters(self, time_period: float, description: str,
                      linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2, scaling_factor: float = 1.0,
                      incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
                      maximal_time_increment: typing.Union[float, str] = 'UNLIMITED'):
        pass

    @typing.overload
    def setParameters(self, time_period: float, description: str,
                      linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2, scaling_factor: float = 1.0,
                      incrementation_type=AbaqusBaseStepIncrementationType.Fixed,
                      increment_size: typing.Union[float, str] = 'ELEMENT_BY_ELEMENT_INCREMENTATION'):
        pass

    def setParameters(self, time_period: float, description: str,
                      linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2,
                      scaling_factor: float = 1.0, incrementation_type=AbaqusBaseStepIncrementationType.Automatic, 
                      *args, **kwargs):
        super(AbaqusExplicitDynamicStep, self).setParameters(AbaqusCalculationMethod.Explicit, time_period, description)
        self.incrementation.setParameters(scaling_factor, incrementation_type, *args, **kwargs)
        self.linear_bulk_viscosity, self.quad_bulk_viscosity = linear_bulk_viscosity, quad_bulk_viscosity



import typing
from enum import IntEnum

from .AbaqusVariable import AbaqusVariable


class AbaqusSubroutineType(IntEnum):
    UMAT = 0
    VUMAT = 1
    FRIC = 2
    VFRIC = 3


class AbaqusSubroutine(AbaqusVariable):
    subroutine_type = AbaqusSubroutineType.FRIC
    subroutine_filepath: str = ''

    n_state_dependent_variables: int = 0
    properties: typing.Iterable = None

    def __init__(self, subroutine_type=AbaqusSubroutineType.FRIC, subroutine_filepath: str = '',
                 n_state_dependent_variables: int = 0, properties: typing.Iterable = 0):
        self.setParameters(subroutine_type, subroutine_filepath, n_state_dependent_variables, properties)

    def setParameters(self, subroutine_type=AbaqusSubroutineType.FRIC, subroutine_filepath: str = '',
                      n_state_dependent_variables: int = 0, properties: typing.Iterable = None):
        if properties is not None:
            properties = tuple((P, ) for P in properties)
        self.subroutine_type, self.subroutine_filepath = subroutine_type, subroutine_filepath
        self.n_state_dependent_variables, self.properties = n_state_dependent_variables, properties

    def subroutineString(self):
        if self.subroutine_type == AbaqusSubroutineType.UMAT:
            return 'UMAT'
        elif self.subroutine_type == AbaqusSubroutineType.VUMAT:
            return 'VUMAT'
        elif self.subroutine_type == AbaqusSubroutineType.FRIC:
            return 'FRIC'
        elif self.subroutine_type == AbaqusSubroutineType.VFRIC:
            return 'VFRIC'



from .AbaqusVariable import *

from .AbaqusGeometry import *
from .AbaqusMaterial import *
from .AbaqusLoad import *
from .AbaqusMesh import *
from .AbaqusSubroutine import *
from .AbaqusStep import *
from .AbaqusOutput import *

from .AbaqusModelBase import *



import os

from abaqus import *
from abaqusConstants import *
from caeModules import *
from driverUtils import executeOnCaeStartup

import numpy as np

# -*- coding: mbcs -*-
#
# Abaqus/CAE Release 2021 replay file
# Internal Version: 2020_03_06-22.50.37 167380
# Run by Hailin on Wed Oct 13 16:22:48 2021
#

# from driverUtils import executeOnCaeGraphicsStartup
# executeOnCaeGraphicsStartup()
#: Executing "onCaeGraphicsStartup()" in the site directory ...
    
SUBROUTINE = 'FRIC'

executeOnCaeStartup()

WORK_DIRECTORY = r'C:\Users\user\OneDrive\Documents\GitHub\CH-Model\CH_model_Python\Model\SoilStructureInterface\Exponential\Abaqus\DirectShear\FRIC'
mdbPath = os.path.join(WORK_DIRECTORY, 'dst.cae')
odbPath = os.path.join(WORK_DIRECTORY, 'Job-1.odb')
os.chdir(WORK_DIRECTORY)

# OPEN OUTPUT DATABASE FILE
odb = session.openOdb(name=odbPath)
session.viewports['Viewport: 1'].setValues(displayedObject=odb)

# TODO CREATE XYDATA
# TODO CONTACT SHEAR1
session.xyDataListFromField(odb=odb, outputPosition=ELEMENT_NODAL, variable=((
    'CSHEAR1', ELEMENT_NODAL), ), elementSets=("INSTANCE-SOIL.SET-SOIL-Z0", ))

# TODO SDV8: gamma_norm
"""session.xyDataListFromField(odb=odb, outputPosition=NODAL, variable=((
    'SDV8     ASSEMBLY_INSTANCE-SOIL_SURFACE-SOIL-Z0/ASSEMBLY_INSTANCE-SOLID_SURFACE-SOLID-Z1', 
    NODAL), ), nodeSets=("INSTANCE-SOIL.SET-SOIL-Z0", ))"""

# TODO U1_RP
session.xyDataListFromField(odb=odb, outputPosition=NODAL, variable=(('U', 
    NODAL, ((COMPONENT, 'U1'), )), ), nodeSets=("INSTANCE-SOLID.SET-RP", ))

# TODO RF1_RP
session.xyDataListFromField(odb=odb, outputPosition=NODAL, variable=(('RF', 
    NODAL, ((COMPONENT, 'RF1'), )), ), nodeSets=("INSTANCE-SOLID.SET-RP", ))

# TODO CSLIP1
"""session.xyDataListFromField(odb=odb, outputPosition=ELEMENT_NODAL, variable=((
    'CSLIP1', ELEMENT_NODAL), ), elementSets=("INSTANCE-SOIL.SET-SOIL-Z0", ))"""

# TODO EPSN_FRIC
"""session.xyDataListFromField(odb=odb, outputPosition=NODAL, variable=((
    'SDV19    ASSEMBLY_INSTANCE-SOIL_SURFACE-SOIL-Z0/ASSEMBLY_INSTANCE-SOLID_SURFACE-SOLID-Z1', 
    NODAL), ), nodeSets=("INSTANCE-SOIL.SET-SOIL-Z0", ))"""

# TODO U3_TOP
session.xyDataListFromField(odb=odb, outputPosition=NODAL, variable=(('U', 
    NODAL, ((COMPONENT, 'U3'), )), ), nodeSets=("INSTANCE-SOIL.SET-SOIL-Z1", ))

# TODO FSLIPQ
session.xyDataListFromField(odb=odb, outputPosition=ELEMENT_NODAL, variable=((
    'FSLIPEQ  ASSEMBLY_INSTANCE-SOIL_SURFACE-SOIL-Z0/ASSEMBLY_INSTANCE-SOLID_SURFACE-SOLID-Z1', 
    ELEMENT_NODAL), ), elementSets=("INSTANCE-SOIL.SET-SOIL-Y0", ))

CSHEAR1, GAMMA_NORM, U1_RP, RF1_RP, CSLIP1, EPSN_FRIC, U3, FSLIPEQ = [], [], [], [], [], [], [], []
for key, xyDataObject in session.xyDataObjects.items():
    if key.startswith('CSHEAR1'):
        CSHEAR1.append(xyDataObject)
    elif key.startswith('SDV8'):
        GAMMA_NORM.append(xyDataObject)
    elif key.startswith('U:U1'):
        U1_RP.append(xyDataObject)
    elif key.startswith('RF:RF1'):
        RF1_RP.append(xyDataObject)
    elif key.startswith('CSLIP1'):
        CSLIP1.append(xyDataObject)
    elif key.startswith('SDV19'):
        EPSN_FRIC.append(xyDataObject)
    elif key.startswith('U:U3'):
        U3.append(xyDataObject)
    elif key.startswith('FSLIPEQ'):
        FSLIPEQ.append(xyDataObject)


def SAVE_DATA_TO_CSV(DATA_LIST, name, ABS = False):
    DATA = np.array(DATA_LIST)
    if ABS:
        DATA_AVG = abs(np.mean(DATA, axis=0))
    else:
        DATA_AVG = np.mean(DATA, axis=0)
    if not os.path.exists(os.path.join(WORK_DIRECTORY, 'DATA')):
        os.mkdir(os.path.join(WORK_DIRECTORY, 'DATA'))
    np.savetxt(os.path.join(WORK_DIRECTORY, ('DATA\\' + name + '.csv')), DATA_AVG, delimiter=',', header='TIME, ' + name)
    
SAVE_DATA_TO_CSV(CSHEAR1, 'CSHEAR1', ABS=True)
# SAVE_DATA_TO_CSV(GAMMA_NORM, 'GAMMA_NORM')
SAVE_DATA_TO_CSV(U1_RP, 'U1_RP')
SAVE_DATA_TO_CSV(RF1_RP, 'RF1_RP')
# SAVE_DATA_TO_CSV(CSLIP1, 'CSLIP1')
# SAVE_DATA_TO_CSV(EPSN_FRIC, 'EPSN_FRIC')
SAVE_DATA_TO_CSV(U3, 'U3_TOP')
SAVE_DATA_TO_CSV(FSLIPEQ, 'FSLIPEQ')



from abaqus import *
from abaqusConstants import *
from caeModules import *
from driverUtils import executeOnCaeStartup

# -*- coding: mbcs -*-
#
# Abaqus/CAE Release 2021 replay file
# Internal Version: 2020_03_06-22.50.37 167380
# Run by Hailin on Wed Oct 13 16:22:48 2021
#

executeOnCaeStartup()

# TODO CALCULATION METHOD
CALCULATION_METHOD = 'STANDARD'  # OR EXPLICIT

# TODO GEOMETRY
SOIL_LENGTH = 0.025
SOIL_WIDTH = 0.025
SOIL_HEIGHT = 0.025
SOLID_LENGTH = 0.05
SOLID_WIDTH = 0.05
SOLID_HEIGHT = 0.05

# TODO MATERIAL
SOIL_ELASTIC_MODULUS = 4e4
SOIL_POISSON_RATIO = 0.0
SOIL_DENSITY = None
SOLID_ELASTIC_MODULUS = 2e8
SOLID_POISSON_RATIO = 0.0
SOLID_DENSITY = None

# TODO MESH
SOIL_MESH_METHOD = 'NUMBER'  # OR 'SIZE'
SOIL_SEEDS_NUMBER = 2
SOIL_SEEDS_SIZE = 0.01
SOLID_MESH_METHOD = 'NUMBER'
SOLID_SEEDS_NUMBER = 2
SOLID_SEEDS_SIZE = 0.01

# TODO STEP
# INITIAL STRESS
INITIAL_STRESS_TIME_PERIOD = 1.0
INITIAL_STRESS_DESCRIPTION = ''
# GENERAL STATIC STEP
INITIAL_STRESS_INCREMENTATION_TYPE = 'FIXED'
INITIAL_STRESS_MAXIMAL_INCREMENTS = 100
INITIAL_STRESS_INITIAL_INCREMENT_SIZE = 1.0
INITIAL_STRESS_MINIMAL_INCREMENT_SIZE = 1e-5
INITIAL_STRESS_MAXIMAL_INCREMENT_SIZE = 1.0
INITIAL_STRESS_FIXED_INCREMENT_SIZE = 0.2
# EXPLICIT DYNAMIC STEP
INITIAL_STRESS_MAXIMAL_TIME_INCREMENT = 'UNLIMITED'
INITIAL_STRESS_INCREMENT_SIZE = 'ELEMENT_BY_ELEMENT_INCREMENTATION'
INITIAL_STRESS_SCALING_FACTOR = 1.0
INITIAL_STRESS_LINEAR_BULK_VISCOSITY = 0.06
INITIAL_STRESS_QUAD_BULK_VISCOSITY = 1.2

# SHEAR
SHEAR_TIME_PERIOD = 1.0
SHEAR_DESCRIPTION = ''
# GENERAL STATIC STEP
SHEAR_INCREMENTATION_TYPE = 'FIXED'
SHEAR_MAXIMAL_INCREMENTS = 100
SHEAR_INITIAL_INCREMENT_SIZE = 1.0
SHEAR_MINIMAL_INCREMENT_SIZE = 1e-5
SHEAR_MAXIMAL_INCREMENT_SIZE = 1.0
SHEAR_FIXED_INCREMENT_SIZE = 0.2
# EXPLICIT DYNAMIC STEP
SHEAR_MAXIMAL_TIME_INCREMENT = 'UNLIMITED'
SHEAR_INCREMENT_SIZE = 'ELEMENT_BY_ELEMENT_INCREMENTATION'
SHEAR_SCALING_FACTOR = 1.0
SHEAR_LINEAR_BULK_VISCOSITY = 0.06
SHEAR_QUAD_BULK_VISCOSITY = 1.2

# TODO LOAD
RP_DISPLACEMENT = [0.006, 0.0, 0.0, 0.0, 0.0, 0.0]
RP_VELOCITY = [0.006, 0.0, 0.0, 0.0, 0.0, 0.0]
VERTICAL_PRESSURE = 310.0
PREDEFINED_STRESS = [-310.0, -310.0, -310.0, 0.0, 0.0, 0.0]

# TODO USER-DEFINED SUBROUTINE
SUBROUTINE = 'FRIC'
SUBROUTINE_PATH = 'FRIC_exponential_CV.for'
N_STATE_DEPENDENT_VARS = 28
FRICTION_PARAMETERS = ((0.0,), (250.0,), (5.19,), (0.89,), (0.147,), (0.424,), (31.2,), (7.57,), (2.06,), (0.46,), (0.5771,), (0.00115,))

# TODO OUTPUT
STANDARD_FREQUENCY = 1
EXPLICIT_NUM_INTERVALS = 50  # FIXME NOT DEFINED
FIELD_OUTPUT_VARIABLES = ['S', 'E', 'LE', 'U', 'RF', 'RT', 'RM', 'P', 'CSTRESS', 'CDISP', 'CFORCE', 'CNAREA', 'CSTATUS'] # STANDARD
# FIELD_OUTPUT_VARIABLES = ['S', 'E', 'LE', 'U', 'V', 'A', 'RF', 'P', 'CSTRESS', 'CFORCE', 'FSLIPR', 'FSLIP', 'PPRESS'] # EXPLICIT
HISTORY_OUTPUT_VARIABLES = ['U1', 'RF1']

# TODO WORK DIRECTORY
WORK_DIRECTORY = r'C:\Users\user\OneDrive\Documents\GitHub\CH-Model\CH_model_Python\SSI\Abaqus\DirectShear'
os.chdir(WORK_DIRECTORY)

# TODO CREATE PARTS
# Soil part
sketch_soil = mdb.models['Model-1'].ConstrainedSketch(name='SKETCH-SOIL', sheetSize=0.1)
sketch_soil.rectangle(point1=(0.0, 0.0), point2=(SOIL_LENGTH, SOIL_WIDTH))
mdb.models['Model-1'].Part(name='PART-SOIL', dimensionality=THREE_D, type=DEFORMABLE_BODY)
mdb.models['Model-1'].parts['PART-SOIL'].BaseSolidExtrude(sketch=sketch_soil, depth=SOIL_HEIGHT)
del mdb.models['Model-1'].sketches['SKETCH-SOIL']
# Solid part
sketch_solid = mdb.models['Model-1'].ConstrainedSketch(name='SKETCH-SOLID', sheetSize=0.1)
sketch_solid.rectangle(point1=(0.0, 0.0), point2=(SOLID_LENGTH, SOLID_WIDTH))
mdb.models['Model-1'].Part(name='PART-SOLID', dimensionality=THREE_D, type=DEFORMABLE_BODY)
mdb.models['Model-1'].parts['PART-SOLID'].BaseSolidExtrude(sketch=sketch_solid, depth=SOLID_HEIGHT)
del mdb.models['Model-1'].sketches['SKETCH-SOLID']

# TODO ASSEMBLY INSTANCE
assembly = mdb.models['Model-1'].rootAssembly
assembly.DatumCsysByDefault(CARTESIAN)
assembly.Instance(name='INSTANCE-SOIL', part=mdb.models['Model-1'].parts['PART-SOIL'], dependent=ON)
assembly.Instance(name='INSTANCE-SOLID', part=mdb.models['Model-1'].parts['PART-SOLID'], dependent=ON)
assembly.translate(instanceList=('INSTANCE-SOIL',), vector=((SOLID_LENGTH - SOIL_LENGTH) / 2,
                                                            (SOLID_WIDTH - SOIL_WIDTH) / 2,
                                                            SOLID_HEIGHT))

# TODO SETS
# Sets of the soil
part_soil = mdb.models['Model-1'].parts['PART-SOIL']
part_soil.Surface(side1Faces=part_soil.faces.findAt(((SOIL_LENGTH / 2, SOIL_WIDTH / 2, 0),), ), name='SURFACE-SOIL-Z0')
part_soil.Surface(side1Faces=part_soil.faces.findAt(((SOIL_LENGTH / 2, SOIL_WIDTH / 2, SOIL_HEIGHT),), ),
                  name='SURFACE-SOIL-Z1')

part_soil.Set(cells=part_soil.cells.findAt(((SOIL_LENGTH / 2, SOIL_WIDTH / 2, SOIL_HEIGHT / 2),), ), name='SET-SOIL')

part_soil.Set(faces=part_soil.faces.findAt(((0, SOIL_WIDTH / 2, SOIL_HEIGHT / 2),), ), name='SET-SOIL-X0')
part_soil.Set(faces=part_soil.faces.findAt(((SOIL_LENGTH / 2, 0, SOIL_HEIGHT / 2),), ), name='SET-SOIL-Y0')
part_soil.Set(faces=part_soil.faces.findAt(((SOIL_LENGTH / 2, SOIL_WIDTH / 2, 0),), ), name='SET-SOIL-Z0')

part_soil.Set(faces=part_soil.faces.findAt(((SOIL_LENGTH, SOIL_WIDTH / 2, SOIL_HEIGHT / 2),), ), name='SET-SOIL-X1')
part_soil.Set(faces=part_soil.faces.findAt(((SOIL_LENGTH / 2, SOIL_WIDTH, SOIL_HEIGHT / 2),), ), name='SET-SOIL-Y1')
part_soil.Set(faces=part_soil.faces.findAt(((SOIL_LENGTH / 2, SOIL_WIDTH / 2, SOIL_HEIGHT),), ), name='SET-SOIL-Z1')

# Sets of the solid
part_solid = mdb.models['Model-1'].parts['PART-SOLID']
part_solid.Surface(side1Faces=part_solid.faces.findAt(((SOLID_LENGTH / 2, SOLID_WIDTH / 2, 0),), ),
                   name='SURFACE-SOLID-Z0')
part_solid.Surface(side1Faces=part_solid.faces.findAt(((SOLID_LENGTH / 2, SOLID_WIDTH / 2, SOLID_HEIGHT),), ),
                   name='SURFACE-SOLID-Z1')

part_solid.Set(cells=part_solid.cells.findAt(((SOLID_LENGTH / 2, SOLID_WIDTH / 2, SOLID_HEIGHT / 2),), ),
               name='SET-SOLID')

part_solid.Set(faces=part_solid.faces.findAt(((0, SOLID_WIDTH / 2, SOLID_HEIGHT / 2),), ), name='SET-SOLID-X0')
part_solid.Set(faces=part_solid.faces.findAt(((SOLID_LENGTH / 2, 0, SOLID_HEIGHT / 2),), ), name='SET-SOLID-Y0')
part_solid.Set(faces=part_solid.faces.findAt(((SOLID_LENGTH / 2, SOLID_WIDTH / 2, 0),), ), name='SET-SOLID-Z0')

part_solid.Set(faces=part_solid.faces.findAt(((SOLID_LENGTH, SOLID_WIDTH / 2, SOLID_HEIGHT / 2),), ),
               name='SET-SOLID-X1')
part_solid.Set(faces=part_solid.faces.findAt(((SOLID_LENGTH / 2, SOLID_WIDTH, SOLID_HEIGHT / 2),), ),
               name='SET-SOLID-Y1')
part_solid.Set(faces=part_solid.faces.findAt(((SOLID_LENGTH / 2, SOLID_WIDTH / 2, SOLID_HEIGHT),), ),
               name='SET-SOLID-Z1')

# TODO MATERIAL
# Materials
mdb.models['Model-1'].Material(name='MATERIAL-SOIL')
mdb.models['Model-1'].materials['MATERIAL-SOIL'].Elastic(table=((SOIL_ELASTIC_MODULUS, SOIL_POISSON_RATIO),))
if SOIL_DENSITY is not None:
    mdb.models['Model-1'].materials['MATERIAL-SOIL'].Density(table=((SOIL_DENSITY,),))
mdb.models['Model-1'].Material(name='MATERIAL-SOLID')
mdb.models['Model-1'].materials['MATERIAL-SOLID'].Elastic(table=((SOLID_ELASTIC_MODULUS, SOLID_POISSON_RATIO),))
if SOLID_DENSITY is not None:
    mdb.models['Model-1'].materials['MATERIAL-SOLID'].Density(table=((SOLID_DENSITY,),))

# Sections
mdb.models['Model-1'].HomogeneousSolidSection(name='SECTION-SOIL', material='MATERIAL-SOIL', thickness=None)
mdb.models['Model-1'].HomogeneousSolidSection(name='SECTION-SOLID', material='MATERIAL-SOLID', thickness=None)

# Sections assignment
part_soil = mdb.models['Model-1'].parts['PART-SOIL']
part_soil.SectionAssignment(region=part_soil.sets['SET-SOIL'], sectionName='SECTION-SOIL', offset=0.0,
                            offsetType=MIDDLE_SURFACE, offsetField='', thicknessAssignment=FROM_SECTION)
part_solid = mdb.models['Model-1'].parts['PART-SOLID']
part_solid.SectionAssignment(region=part_solid.sets['SET-SOLID'], sectionName='SECTION-SOLID', offset=0.0,
                             offsetType=MIDDLE_SURFACE, offsetField='', thicknessAssignment=FROM_SECTION)

# TODO STEP
if CALCULATION_METHOD == 'STANDARD':
    mdb.models['Model-1'].StaticStep(name='INITIAL-STRESS', previous='Initial',
                                     maxNumInc=INITIAL_STRESS_MAXIMAL_INCREMENTS,
                                     initialInc=INITIAL_STRESS_INITIAL_INCREMENT_SIZE,
                                     minInc=INITIAL_STRESS_MINIMAL_INCREMENT_SIZE,
                                     maxInc=INITIAL_STRESS_MAXIMAL_INCREMENT_SIZE,
                                     matrixSolver=DIRECT, matrixStorage=UNSYMMETRIC)
    mdb.models['Model-1'].StaticStep(name='SHEAR', previous='INITIAL-STRESS',
                                     maxNumInc=SHEAR_MAXIMAL_INCREMENTS,
                                     initialInc=SHEAR_INITIAL_INCREMENT_SIZE,
                                     minInc=SHEAR_MINIMAL_INCREMENT_SIZE,
                                     maxInc=SHEAR_MAXIMAL_INCREMENT_SIZE,
                                     matrixSolver=DIRECT, matrixStorage=UNSYMMETRIC)
else:
    if INITIAL_STRESS_INCREMENTATION_TYPE == 'AUTOMATIC':
        if INITIAL_STRESS_MAXIMAL_TIME_INCREMENT == 'UNLIMITED':
            mdb.models['Model-1'].ExplicitDynamicsStep(name='INITIAL-STRESS', previous='Initial',
                                                       timePeriod=INITIAL_STRESS_TIME_PERIOD,
                                                       linearBulkViscosity=INITIAL_STRESS_LINEAR_BULK_VISCOSITY,
                                                       quadBulkViscosity=INITIAL_STRESS_QUAD_BULK_VISCOSITY,
                                                       improvedDtMethod=ON,
                                                       timeIncrementationMethod=AUTOMATIC_GLOBAL)
        else:
            mdb.models['Model-1'].ExplicitDynamicsStep(name='INITIAL-STRESS', previous='Initial',
                                                       timePeriod=INITIAL_STRESS_TIME_PERIOD,
                                                       linearBulkViscosity=INITIAL_STRESS_LINEAR_BULK_VISCOSITY,
                                                       quadBulkViscosity=INITIAL_STRESS_QUAD_BULK_VISCOSITY,
                                                       improvedDtMethod=ON,
                                                       maxIncrement=INITIAL_STRESS_MAXIMAL_TIME_INCREMENT,
                                                       timeIncrementationMethod=AUTOMATIC_GLOBAL)
    else:
        if INITIAL_STRESS_INCREMENT_SIZE == 'ELEMENT_BY_ELEMENT_INCREMENTATION':
            mdb.models['Model-1'].ExplicitDynamicsStep(name='INITIAL-STRESS', previous='Initial',
                                                       timePeriod=INITIAL_STRESS_TIME_PERIOD,
                                                       linearBulkViscosity=INITIAL_STRESS_LINEAR_BULK_VISCOSITY,
                                                       quadBulkViscosity=INITIAL_STRESS_QUAD_BULK_VISCOSITY,
                                                       improvedDtMethod=ON,
                                                       timeIncrementationMethod=FIXED_EBE)
        else:
            mdb.models['Model-1'].ExplicitDynamicsStep(name='INITIAL-STRESS', previous='Initial',
                                                       timePeriod=INITIAL_STRESS_TIME_PERIOD,
                                                       linearBulkViscosity=INITIAL_STRESS_LINEAR_BULK_VISCOSITY,
                                                       quadBulkViscosity=INITIAL_STRESS_QUAD_BULK_VISCOSITY,
                                                       improvedDtMethod=ON,
                                                       timeIncrementationMethod=FIXED_USER_DEFINED_INC,
                                                       userDefinedInc=INITIAL_STRESS_INCREMENT_SIZE)
    if SHEAR_INCREMENTATION_TYPE == 'AUTOMATIC':
        if SHEAR_MAXIMAL_TIME_INCREMENT == 'UNLIMITED':
            mdb.models['Model-1'].ExplicitDynamicsStep(name='SHEAR', previous='INITIAL-STRESS',
                                                       timePeriod=SHEAR_TIME_PERIOD,
                                                       linearBulkViscosity=SHEAR_LINEAR_BULK_VISCOSITY,
                                                       quadBulkViscosity=SHEAR_QUAD_BULK_VISCOSITY, improvedDtMethod=ON,
                                                       timeIncrementationMethod=AUTOMATIC_GLOBAL)
        else:
            mdb.models['Model-1'].ExplicitDynamicsStep(name='SHEAR', previous='INITIAL-STRESS',
                                                       timePeriod=SHEAR_TIME_PERIOD,
                                                       linearBulkViscosity=SHEAR_LINEAR_BULK_VISCOSITY,
                                                       quadBulkViscosity=SHEAR_QUAD_BULK_VISCOSITY, improvedDtMethod=ON,
                                                       maxIncrement=SHEAR_MAXIMAL_TIME_INCREMENT,
                                                       timeIncrementationMethod=AUTOMATIC_GLOBAL)
    else:
        if SHEAR_INCREMENT_SIZE == 'ELEMENT_BY_ELEMENT_INCREMENTATION':
            mdb.models['Model-1'].ExplicitDynamicsStep(name='SHEAR', previous='INITIAL-STRESS',
                                                       timePeriod=SHEAR_TIME_PERIOD,
                                                       linearBulkViscosity=SHEAR_LINEAR_BULK_VISCOSITY,
                                                       quadBulkViscosity=SHEAR_QUAD_BULK_VISCOSITY, improvedDtMethod=ON,
                                                       timeIncrementationMethod=FIXED_EBE)
        else:
            mdb.models['Model-1'].ExplicitDynamicsStep(name='SHEAR', previous='INITIAL-STRESS',
                                                       timePeriod=SHEAR_TIME_PERIOD,
                                                       linearBulkViscosity=SHEAR_LINEAR_BULK_VISCOSITY,
                                                       quadBulkViscosity=SHEAR_QUAD_BULK_VISCOSITY, improvedDtMethod=ON,
                                                       timeIncrementationMethod=FIXED_USER_DEFINED_INC,
                                                       userDefinedInc=SHEAR_INCREMENT_SIZE)

# TODO CONTACT
# Contact property
mdb.models['Model-1'].ContactProperty('IntProp-1')
mdb.models['Model-1'].interactionProperties['IntProp-1'].NormalBehavior(pressureOverclosure=HARD, allowSeparation=ON,
                                                                        constraintEnforcementMethod=DEFAULT)
mdb.models['Model-1'].interactionProperties['IntProp-1'].TangentialBehavior(
    formulation=USER_DEFINED, nStateDependentVars=N_STATE_DEPENDENT_VARS,
    useProperties=ON, table=FRICTION_PARAMETERS)

# Contact assignment
assembly = mdb.models['Model-1'].rootAssembly
region_solid = assembly.instances['INSTANCE-SOLID'].surfaces['SURFACE-SOLID-Z1']
region_soil = assembly.instances['INSTANCE-SOIL'].surfaces['SURFACE-SOIL-Z0']
if CALCULATION_METHOD == 'STANDARD':
    mdb.models['Model-1'].SurfaceToSurfaceContactStd(name='Int-1', createStepName='Initial', master=region_solid,
                                                     slave=region_soil, sliding=FINITE, thickness=ON,
                                                     interactionProperty='IntProp-1', adjustMethod=NONE,
                                                     initialClearance=OMIT, datumAxis=None, clearanceRegion=None)
else:  # EXPLICIT
    mdb.models['Model-1'].SurfaceToSurfaceContactExp(name='Int-1', createStepName='Initial', master=region_solid,
                                                     slave=region_soil, sliding=FINITE, initialClearance=OMIT,
                                                     interactionProperty='IntProp-1', datumAxis=None,
                                                     clearanceRegion=None,
                                                     mechanicalConstraint=KINEMATIC)

# TODO LOAD
# Vertical pressure
region_pressure = mdb.models['Model-1'].rootAssembly.instances['INSTANCE-SOIL'].surfaces['SURFACE-SOIL-Z1']
mdb.models['Model-1'].Pressure(name='SOIL-VERTICAL-PRESSURE', createStepName='INITIAL-STRESS', region=region_pressure,
                               distributionType=UNIFORM, field='', magnitude=VERTICAL_PRESSURE, amplitude=UNSET)
# Predefined stress
region_predefined_stress = mdb.models['Model-1'].rootAssembly.instances['INSTANCE-SOIL'].sets['SET-SOIL']
mdb.models['Model-1'].Stress(name='SOIL-PREDEFINED-STRESS', region=region_predefined_stress, distributionType=UNIFORM,
                             sigma11=PREDEFINED_STRESS[0], sigma22=PREDEFINED_STRESS[1], sigma33=PREDEFINED_STRESS[2],
                             sigma12=PREDEFINED_STRESS[3], sigma13=PREDEFINED_STRESS[4], sigma23=PREDEFINED_STRESS[5])

# Confining pressure
# Reference point
mdb.models['Model-1'].parts['PART-SOLID'].ReferencePoint(point=(0.0, 0.0, 0.0))
RP = mdb.models['Model-1'].parts['PART-SOLID'].referencePoints[11]
mdb.models['Model-1'].parts['PART-SOLID'].Set(referencePoints=(RP,), name='SET-RP')

# Rigid body
region1 = mdb.models['Model-1'].rootAssembly.instances['INSTANCE-SOLID'].sets['SET-RP']
region2 = mdb.models['Model-1'].rootAssembly.instances['INSTANCE-SOLID'].sets['SET-SOLID']
mdb.models['Model-1'].RigidBody(name='CONSTRAINT-SOLID', refPointRegion=region1, bodyRegion=region2)

# BC-RP INITIAL: FIXED; SHEAR: U1 = 0.006
region_RP = mdb.models['Model-1'].rootAssembly.instances['INSTANCE-SOLID'].sets['SET-RP']
if CALCULATION_METHOD == 'STANDARD':
    mdb.models['Model-1'].DisplacementBC(name='BC-RP', createStepName='Initial',
                                         region=region_RP, u1=SET, u2=SET, u3=SET, ur1=SET, ur2=SET, ur3=SET,
                                         amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)
    mdb.models['Model-1'].boundaryConditions['BC-RP'].setValuesInStep(
        stepName='SHEAR', u1=RP_DISPLACEMENT[0], u2=RP_DISPLACEMENT[1], u3=RP_DISPLACEMENT[2],
        ur1=RP_DISPLACEMENT[3], ur2=RP_DISPLACEMENT[4], ur3=RP_DISPLACEMENT[5])
else:
    mdb.models['Model-1'].VelocityBC(name='BC-RP', createStepName='Initial',
                                     region=region_RP, v1=SET, v2=SET, v3=SET, vr1=SET, vr2=SET, vr3=SET,
                                     amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)
    mdb.models['Model-1'].boundaryConditions['BC-RP'].setValuesInStep(
        stepName='SHEAR', v1=RP_VELOCITY[0], v2=RP_VELOCITY[1], v3=RP_VELOCITY[2],
        vr1=RP_VELOCITY[3], vr2=RP_VELOCITY[4], vr3=RP_VELOCITY[5])

# BC-SOIL-X0 INITIAL: U1 = 0
assembly = mdb.models['Model-1'].rootAssembly
region_X0 = assembly.instances['INSTANCE-SOIL'].sets['SET-SOIL-X0']
if CALCULATION_METHOD == 'STANDARD':
    mdb.models['Model-1'].DisplacementBC(name='BC-SOIL-X0', createStepName='Initial',
                                         region=region_X0, u1=0.0, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET,
                                         amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, fieldName='',
                                         localCsys=None)
else:
    mdb.models['Model-1'].VelocityBC(name='BC-SOIL-X0', createStepName='Initial',
                                     region=region_X0, v1=0.0, v2=UNSET, v3=UNSET, vr1=UNSET, vr2=UNSET, vr3=UNSET,
                                     amplitude=UNSET, distributionType=UNIFORM, fieldName='',
                                     localCsys=None)

# BC-SOIL-X1 INITIAL: U1 = 0
region_X1 = assembly.instances['INSTANCE-SOIL'].sets['SET-SOIL-X1']
if CALCULATION_METHOD == 'STANDARD':
    mdb.models['Model-1'].DisplacementBC(name='BC-SOIL-X1', createStepName='Initial',
                                         region=region_X1, u1=0.0, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET,
                                         amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, fieldName='',
                                         localCsys=None)
else:
    mdb.models['Model-1'].VelocityBC(name='BC-SOIL-X1', createStepName='Initial',
                                     region=region_X1, v1=0.0, v2=UNSET, v3=UNSET, vr1=UNSET, vr2=UNSET, vr3=UNSET,
                                     amplitude=UNSET, distributionType=UNIFORM, fieldName='',
                                     localCsys=None)

# BC-SOIL-Y0 INITIAL: U2 = 0
region_Y0 = assembly.instances['INSTANCE-SOIL'].sets['SET-SOIL-Y0']
if CALCULATION_METHOD == 'STANDARD':
    mdb.models['Model-1'].DisplacementBC(name='BC-SOIL-Y0', createStepName='Initial',
                                         region=region_Y0, u1=UNSET, u2=0.0, u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET,
                                         amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, fieldName='',
                                         localCsys=None)
else:
    mdb.models['Model-1'].VelocityBC(name='BC-SOIL-Y0', createStepName='Initial',
                                     region=region_Y0, v1=UNSET, v2=0.0, v3=UNSET, vr1=UNSET, vr2=UNSET, vr3=UNSET,
                                     amplitude=UNSET, distributionType=UNIFORM, fieldName='',
                                     localCsys=None)

# BC-SOIL-Y1 INITIAL: U2 = 0
region_Y1 = assembly.instances['INSTANCE-SOIL'].sets['SET-SOIL-Y1']
if CALCULATION_METHOD == 'STANDARD':
    mdb.models['Model-1'].DisplacementBC(name='BC-SOIL-Y1', createStepName='Initial',
                                         region=region_Y1, u1=UNSET, u2=0.0, u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET,
                                         amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, fieldName='',
                                         localCsys=None)
else:
    mdb.models['Model-1'].VelocityBC(name='BC-SOIL-Y1', createStepName='Initial',
                                     region=region_Y1, v1=UNSET, v2=0.0, v3=UNSET, vr1=UNSET, vr2=UNSET, vr3=UNSET,
                                     amplitude=UNSET, distributionType=UNIFORM, fieldName='',
                                     localCsys=None)

# TODO MESH
# Element type
# Mesh soil
elemType1 = mesh.ElemType(elemCode=C3D8R, elemLibrary=STANDARD, kinematicSplit=AVERAGE_STRAIN,
                          secondOrderAccuracy=OFF, hourglassControl=ENHANCED, distortionControl=DEFAULT)
elemType2 = mesh.ElemType(elemCode=C3D6, elemLibrary=STANDARD)
elemType3 = mesh.ElemType(elemCode=C3D4, elemLibrary=STANDARD)

p = mdb.models['Model-1'].parts['PART-SOIL']
cells = p.cells.findAt(((SOIL_LENGTH / 2, SOIL_WIDTH / 2, SOIL_HEIGHT / 2),), )
p.setElementType(regions=(cells,), elemTypes=(elemType1, elemType2, elemType3))

if SOIL_MESH_METHOD == 'SIZE':
    p.seedPart(size=SOIL_SEEDS_SIZE, deviationFactor=0.1, minSizeFactor=0.1)
else:
    p.seedEdgeByNumber(edges=p.edges, number=SOIL_SEEDS_NUMBER, constraint=FINER)

mdb.models['Model-1'].parts['PART-SOIL'].generateMesh()

# Mesh solid
if CALCULATION_METHOD == 'STANDARD':
    elemType1 = mesh.ElemType(elemCode=C3D8R, elemLibrary=STANDARD, kinematicSplit=AVERAGE_STRAIN,
                              secondOrderAccuracy=OFF, hourglassControl=ENHANCED, distortionControl=DEFAULT)
else:
    elemType1 = mesh.ElemType(elemCode=C3D8R, elemLibrary=STANDARD, kinematicSplit=AVERAGE_STRAIN,
                              secondOrderAccuracy=OFF, hourglassControl=RELAX_STIFFNESS, distortionControl=DEFAULT)

elemType2 = mesh.ElemType(elemCode=C3D6, elemLibrary=STANDARD)
elemType3 = mesh.ElemType(elemCode=C3D4, elemLibrary=STANDARD)
p = mdb.models['Model-1'].parts['PART-SOLID']
cells = p.cells.findAt(((SOLID_LENGTH / 2, SOLID_WIDTH / 2, SOLID_HEIGHT / 2),), )
p.setElementType(regions=(cells,), elemTypes=(elemType1, elemType2, elemType3))

if SOLID_MESH_METHOD == 'SIZE':
    p.seedPart(size=SOLID_SEEDS_SIZE, deviationFactor=0.1, minSizeFactor=0.1)
else:
    p.seedEdgeByNumber(edges=p.edges, number=SOLID_SEEDS_NUMBER, constraint=FINER)

mdb.models['Model-1'].parts['PART-SOLID'].generateMesh()

# TODO FIELD OUTPUT REQUEST FIXME Modify the input file to add SDV variable of the contact
if CALCULATION_METHOD == 'STANDARD':
    mdb.models['Model-1'].fieldOutputRequests['F-Output-1'].setValues(
        variables=tuple(FIELD_OUTPUT_VARIABLES),
        frequency=STANDARD_FREQUENCY)
else:  # EXPLICIT
    mdb.models['Model-1'].fieldOutputRequests['F-Output-1'].setValues(
        variables=tuple(FIELD_OUTPUT_VARIABLES),
        numIntervals=EXPLICIT_NUM_INTERVALS)
    pass

# TODO HISTORY OUTPUT REQUEST
regionDef = mdb.models['Model-1'].rootAssembly.allInstances['INSTANCE-SOLID'].sets['SET-RP']
mdb.models['Model-1'].HistoryOutputRequest(name='H-Output-1', createStepName='SHEAR',
                                           variables=tuple(HISTORY_OUTPUT_VARIABLES), region=regionDef,
                                           sectionPoints=DEFAULT, rebar=EXCLUDE)

# TODO JOB
mdb.Job(name='Job-1', model='Model-1', description='', type=ANALYSIS,
        atTime=None, waitMinutes=0, waitHours=0, queue=None, memory=90,
        memoryUnits=PERCENTAGE, getMemoryFromAnalysis=True,
        explicitPrecision=DOUBLE, nodalOutputPrecision=SINGLE, echoPrint=OFF,
        modelPrint=OFF, contactPrint=OFF, historyPrint=OFF,
        userSubroutine=SUBROUTINE_PATH,
        scratch='', resultsFormat=ODB, multiprocessingMode=DEFAULT, numCpus=1,
        numGPUs=0)

# TODO MODIFY INPUT FILE: ADD SDV OUTPUT VARIABLES
if CALCULATION_METHOD == 'STANDARD':
    mdb.models['Model-1'].keywordBlock.synchVersions(storeNodesAndElements=False)
    mdb.models['Model-1'].keywordBlock.replace(101, """
    *Contact Output
    CDISP, CFORCE, CNAREA, CSTATUS, CSTRESS, SDV""")
    mdb.models['Model-1'].keywordBlock.replace(115, """
    *Contact Output
    CDISP, CFORCE, CNAREA, CSTATUS, CSTRESS, SDV""")
else:  # EXPLICIT
    pass

# TODO WRITE INPUT FILE
mdb.jobs['Job-1'].writeInput()

# TODO SAVE CAE MODEL TO FILE
mdb.saveAs(pathName='dst.cae')

# TODO SUBMIT JOB
# mdb.jobs['Job-1'].submit(consistencyChecking=OFF)

# OPEN OUTPUT DATABASE FILE
# mdb.jobs['Job-1'].waitForCompletion()
# odb = session.openOdb(name=os.path.join(WORK_DIRECTORY, 'Job-1.odb'))
# session.viewports['Viewport: 1'].setValues(displayedObject=odb)
